import logging
import os
from datetime import timedelta, datetime

from landscape.lib.timestamp import to_timestamp
from landscape.lib.jiffies import detect_jiffies

# FIXME: It'd be nice to avoid having library code which depends on
#        landscape-specific modules.
from landscape.monitor.computeruptime import BootTimes, get_uptime


class ProcessInformation(object):
    """
    @param proc_dir: The directory to use for process information.
    @param jiffies: The value to use for jiffies per second.
    @param boot_time: An alternate value to use for the last boot time.  If
        None, the system last boot time will be used.
    @param uptime: The uptime value to use (for unit tests only).
    """

    def __init__(self, proc_dir="/proc", jiffies=None, boot_time=None,
                 uptime=None):
        if boot_time is None:
            boot_time = BootTimes().get_last_boot_time()
        if boot_time is not None:
            boot_time = datetime.utcfromtimestamp(boot_time)
        self._boot_time = boot_time
        self._proc_dir = proc_dir
        self._jiffies_per_sec = jiffies or detect_jiffies()
        self._uptime = uptime

    def get_all_process_info(self):
        """Get process information for all processes on the system."""
        for filename in os.listdir(self._proc_dir):
            try:
                process_id = int(filename)
            except ValueError:
                continue
            process_info = self.get_process_info(process_id)
            if process_info:
                yield process_info

    def get_process_info(self, process_id):
        """
        Parse the /proc/<pid>/cmdline and /proc/<pid>/status files for
        information about the running process with process_id.

        The /proc filesystem doesn't behave like ext2, open files can disappear
        during the read process.
        """
        cmd_line_name = ""
        process_dir = os.path.join(self._proc_dir, str(process_id))
        process_info = {"pid": process_id}

        try:
            file = open(os.path.join(process_dir, "cmdline"), "r")
            try:
                # cmdline is a \0 separated list of strings
                # We take the first, and then strip off the path, leaving
                # us with the basename.
                cmd_line = file.readline()
                cmd_line_name = os.path.basename(cmd_line.split("\0")[0])
            finally:
                file.close()

            file = open(os.path.join(process_dir, "status"), "r")
            try:
                for line in file:
                    parts = line.split(":", 1)
                    if parts[0] == "Name":
                        process_info["name"] = (cmd_line_name.strip() or
                                                parts[1].strip())
                    elif parts[0] == "State":
                        state = parts[1].strip()
                        # In Lucid, capital T is used for both tracing stop
                        # and stopped. Starting with Natty, lowercase t is
                        # used for tracing stop.
                        if state == "T (tracing stop)":
                            state = state.lower()
                        process_info["state"] = state[0]
                    elif parts[0] == "Uid":
                        value_parts = parts[1].split()
                        process_info["uid"] = int(value_parts[0])
                    elif parts[0] == "Gid":
                        value_parts = parts[1].split()
                        process_info["gid"] = int(value_parts[0])
                    elif parts[0] == "VmSize":
                        value_parts = parts[1].split()
                        process_info["vm-size"] = int(value_parts[0])
                        break
            finally:
                file.close()

            file = open(os.path.join(process_dir, "stat"), "r")
            try:
                # These variable names are lifted directly from proc(5)
                # utime: The number of jiffies that this process has been
                #        scheduled in user mode.
                # stime: The number of jiffies that this process has been
                #        scheduled in kernel mode.
                # cutime: The number of jiffies that this process's waited-for
                #         children have been scheduled in user mode.
                # cstime: The number of jiffies that this process's waited-for
                #         children have been scheduled in kernel mode.
                parts = file.read().split()
                start_time = int(parts[21])
                utime = int(parts[13])
                stime = int(parts[14])
                uptime = self._uptime or get_uptime()
                pcpu = calculate_pcpu(utime, stime, uptime,
                                      start_time, self._jiffies_per_sec)
                process_info["percent-cpu"] = pcpu
                delta = timedelta(0, start_time // self._jiffies_per_sec)
                if self._boot_time is None:
                    logging.warning(
                        "Skipping process (PID %s) without boot time.")
                    return None
                process_info["start-time"] = to_timestamp(
                    self._boot_time + delta)
            finally:
                file.close()

        except IOError:
            # Handle the race that happens when we find a process
            # which terminates before we open the stat file.
            return None

        assert("pid" in process_info and "state" in process_info
               and "name" in process_info and "uid" in process_info
               and "gid" in process_info and "start-time" in process_info)
        return process_info


def calculate_pcpu(utime, stime, uptime, start_time, hertz):
    """
    Implement ps' algorithm to calculate the percentage cpu utilisation for a
    process.::

    unsigned long long total_time;   /* jiffies used by this process */
    unsigned pcpu = 0;               /* scaled %cpu, 99 means 99% */
    unsigned long long seconds;      /* seconds of process life */
    total_time = pp->utime + pp->stime;
    if(include_dead_children) total_time += (pp->cutime + pp->cstime);
    seconds = seconds_since_boot - pp->start_time / hertz;
    if(seconds) pcpu = (total_time * 100ULL / hertz) / seconds;
    if (pcpu > 99U) pcpu = 99U;
    return snprintf(outbuf, COLWID, "%2u", pcpu);
    """
    pcpu = 0
    total_time = utime + stime
    seconds = uptime - (start_time / hertz)
    if seconds:
        pcpu = total_time * 100 / hertz / seconds
    return round(max(min(pcpu, 99.0), 0), 1)



"""
Handle client registration against the server.

When the service is started for the first time it connects to the server
as a new client without providing any identification credentials, and the
server replies with the available registration mechanisms. At this point
the machinery in this module will notice that we have no identification
credentials yet and that the server accepts registration messages, so it
will craft an appropriate one and send it out.
"""
import logging

from twisted.internet.defer import Deferred

from landscape.lib.juju import get_juju_info
from landscape.lib.tag import is_valid_tag_list
from landscape.lib.network import get_fqdn
from landscape.lib.vm_info import get_vm_info, get_container_info
from landscape.lib.versioning import is_version_higher


class InvalidCredentialsError(Exception):
    """
    Raised when an invalid account title and/or registration key
    is used with L{RegistrationManager.register}.
    """


def persist_property(name):

    def get(self):
        return self._persist.get(name)

    def set(self, value):
        self._persist.set(name, value)

    return property(get, set)


def config_property(name):

    def get(self):
        return getattr(self._config, name)

    return property(get)


class Identity(object):
    """Maintains details about the identity of this Landscape client.

    @ivar secure_id: A server-provided ID for secure message exchange.
    @ivar insecure_id: Non-secure server-provided ID, mainly used with
        the ping server.
    @ivar computer_title: See L{BrokerConfiguration}.
    @ivar account_name: See L{BrokerConfiguration}.
    @ivar registration_password: See L{BrokerConfiguration}.
    @ivar tags: See L{BrokerConfiguration}

    @param config: A L{BrokerConfiguration} object, used to set the
        C{computer_title}, C{account_name} and C{registration_password}
        instance variables.
    """

    secure_id = persist_property("secure-id")
    insecure_id = persist_property("insecure-id")

    computer_title = config_property("computer_title")
    account_name = config_property("account_name")
    registration_key = config_property("registration_key")
    tags = config_property("tags")
    access_group = config_property("access_group")

    def __init__(self, config, persist):
        self._config = config
        self._persist = persist.root_at("registration")


class RegistrationHandler(object):
    """
    An object from which registration can be requested of the server,
    and which will handle forced ID changes from the server.

    L{register} should be used to perform initial registration.
    """

    def __init__(self, config, identity, reactor, exchange, pinger,
                 message_store, fetch_async=None):
        self._config = config
        self._identity = identity
        self._reactor = reactor
        self._exchange = exchange
        self._pinger = pinger
        self._message_store = message_store
        self._reactor.call_on("run", self._get_juju_data)
        self._reactor.call_on("pre-exchange", self._handle_pre_exchange)
        self._reactor.call_on("exchange-done", self._handle_exchange_done)
        self._exchange.register_message("set-id", self._handle_set_id)
        self._exchange.register_message("unknown-id", self._handle_unknown_id)
        self._exchange.register_message("registration",
                                        self._handle_registration)
        self._should_register = None
        self._fetch_async = fetch_async
        self._ec2_data = None
        self._juju_data = None

    def should_register(self):
        id = self._identity
        if id.secure_id:
            return False

        return bool(id.computer_title and id.account_name
                    and self._message_store.accepts("register"))

    def register(self):
        """
        Attempt to register with the Landscape server.

        @return: A L{Deferred} which will either be fired with None if
            registration was successful or will fail with an
            L{InvalidCredentialsError} if not.
        """
        self._identity.secure_id = None
        self._identity.insecure_id = None
        result = RegistrationResponse(self._reactor).deferred
        self._exchange.exchange()
        return result

    def _get_juju_data(self):
        """Load Juju information."""
        juju_info = get_juju_info(self._config)
        if juju_info is None:
            return None
        self._juju_data = juju_info

    def _handle_exchange_done(self):
        """Registered handler for the C{"exchange-done"} event.

        If we are not registered yet, schedule another message exchange.

        The first exchange made us accept the message type "register", so
        the next "pre-exchange" event will make L{_handle_pre_exchange}
        queue a registration message for delivery.
        """
        if self.should_register() and not self._should_register:
            self._exchange.exchange()

    def _handle_pre_exchange(self):
        """
        An exchange is about to happen.  If we don't have a secure id already
        set, and we have the needed information available, queue a registration
        message with the server.
        """
        # The point of storing this flag is that if we should *not* register
        # now, and then after the exchange we *should*, we schedule an urgent
        # exchange again.  Without this flag we would just spin trying to
        # connect to the server when something is clearly preventing the
        # registration.
        self._should_register = self.should_register()
        if not self._should_register:
            return

        # These are just to shorten the code.
        identity = self._identity
        account_name = identity.account_name

        if not account_name:
            self._reactor.fire("registration-failed")
            return

        tags = identity.tags
        group = identity.access_group
        registration_key = identity.registration_key

        self._message_store.delete_all_messages()

        if not is_valid_tag_list(tags):
            tags = None
            logging.error("Invalid tags provided for registration.")

        message = {"type": "register",
                   "hostname": get_fqdn(),
                   "account_name": account_name,
                   "computer_title": identity.computer_title,
                   "registration_password": identity.registration_key,
                   "tags": tags,
                   "container-info": get_container_info(),
                   "vm-info": get_vm_info()}

        if group:
            message["access_group"] = group

        server_api = self._message_store.get_server_api()
        # If we have juju data to send and if the server is recent enough to
        # know how to handle juju data, then we include it in the registration
        # message. We want to trigger the 3.3 server handler because client
        # version 14.01 has a different format for the juju-info field,
        # so this makes sure that the correct schema is used by the server
        # when validating our message.
        if self._juju_data and is_version_higher(server_api, "3.3"):
            message["juju-info"] = {
                "environment-uuid": self._juju_data["environment-uuid"],
                "api-addresses": self._juju_data["api-addresses"],
                "machine-id": self._juju_data["machine-id"]}

        # The computer is a normal computer, possibly a container.
        with_word = "with" if bool(registration_key) else "without"
        with_tags = "and tags %s " % tags if tags else ""
        with_group = "in access group '%s' " % group if group else ""

        logging.info(
            u"Queueing message to register with account %r %s%s"
            "%s a password." % (
                account_name, with_group, with_tags, with_word))
        self._exchange.send(message)

    def _handle_set_id(self, message):
        """Registered handler for the C{"set-id"} event.

        Record and start using the secure and insecure IDs from the given
        message.

        Fire C{"registration-done"} and C{"resynchronize-clients"}.
        """
        id = self._identity
        if id.secure_id:
            logging.info("Overwriting secure_id with '%s'" % id.secure_id)

        id.secure_id = message.get("id")
        id.insecure_id = message.get("insecure-id")
        logging.info("Using new secure-id ending with %s for account %s.",
                     id.secure_id[-10:], id.account_name)
        logging.debug("Using new secure-id: %s", id.secure_id)
        self._reactor.fire("registration-done")
        self._reactor.fire("resynchronize-clients")

    def _handle_registration(self, message):
        if message["info"] == "unknown-account":
            self._reactor.fire("registration-failed")

    def _handle_unknown_id(self, message):
        id = self._identity
        clone = message.get("clone-of")
        if clone is None:
            logging.info("Client has unknown secure-id for account %s."
                         % id.account_name)
        else:
            logging.info("Client is clone of computer %s" % clone)
            # Set a new computer title so when a registration request will be
            # made, the pending computer UI will indicate that this is a clone
            # of another computer. There's no need to persist the changes since
            # a new registration will be requested immediately.
            if clone == self._config.computer_title:
                title = "%s (clone)" % self._config.computer_title
            else:
                title = "%s (clone of %s)" % (self._config.computer_title,
                                              clone)
            self._config.computer_title = title
        id.secure_id = None
        id.insecure_id = None


class RegistrationResponse(object):
    """A helper for dealing with the response of a single registration request.

    @ivar deferred: The L{Deferred} that will be fired as per
        L{RegistrationHandler.register}.
    """

    def __init__(self, reactor):
        self._reactor = reactor
        self._done_id = reactor.call_on("registration-done", self._done)
        self._failed_id = reactor.call_on("registration-failed", self._failed)
        self.deferred = Deferred()

    def _cancel_calls(self):
        self._reactor.cancel_call(self._done_id)
        self._reactor.cancel_call(self._failed_id)

    def _done(self):
        self.deferred.callback(None)
        self._cancel_calls()

    def _failed(self):
        self.deferred.errback(InvalidCredentialsError())
        self._cancel_calls()



import logging
import signal

from twisted.internet import reactor
from twisted.internet.task import deferLater

from landscape.reactor import FakeReactor
from landscape.deployment import Configuration
from landscape.service import LandscapeService
from landscape.tests.helpers import LandscapeTest


class TestComponent(object):
    name = "monitor"


class TestService(LandscapeService):
    service_name = TestComponent.name


class LandscapeServiceTest(LandscapeTest):

    def setUp(self):
        super(LandscapeServiceTest, self).setUp()
        self.config = Configuration()
        self.config.data_path = self.makeDir()
        self.makeDir(path=self.config.sockets_path)
        self.reactor = FakeReactor()
        signal.signal(signal.SIGUSR1, signal.SIG_DFL)

    def tearDown(self):
        super(LandscapeServiceTest, self).tearDown()
        signal.signal(signal.SIGUSR1, signal.SIG_DFL)

    def test_create_persist(self):
        """
        If a {persist_filename} attribute is defined, a L{Persist} with that
        filename will be created.
        """

        class PersistService(TestService):
            persist_filename = self.makeFile(content="")

        service = PersistService(self.config)
        self.assertEqual(service.persist.filename, service.persist_filename)

    def test_no_persist_without_filename(self):
        """
        If no {persist_filename} attribute is defined, no C{persist} attribute
        will be available.
        """
        service = TestService(self.config)
        self.assertFalse(hasattr(service, "persist"))

    def test_install_bpickle_dbus(self):
        """
        A L{LandscapeService} installs the DBus extensions of bpickle.
        """
        dbus_mock = self.mocker.replace("landscape.lib.bpickle_dbus.install")
        dbus_mock()
        self.mocker.replay()
        TestService(self.config)

    def test_usr1_rotates_logs(self):
        """
        SIGUSR1 should cause logs to be reopened.
        """
        logging.getLogger().addHandler(logging.FileHandler(self.makeFile()))
        # Store the initial set of handlers
        original_streams = [handler.stream for handler in
                            logging.getLogger().handlers if
                            isinstance(handler, logging.FileHandler)]

        # Instantiating LandscapeService should register the handler
        TestService(self.config)
        # We'll call it directly
        handler = signal.getsignal(signal.SIGUSR1)
        self.assertTrue(handler)
        handler(None, None)

        def check(ign):
            new_streams = [handler.stream for handler in
                           logging.getLogger().handlers if
                           isinstance(handler, logging.FileHandler)]

            for stream in new_streams:
                self.assertTrue(stream not in original_streams)

        # We need to give some room for the callFromThread to run
        d = deferLater(reactor, 0, lambda: None)
        return d.addCallback(check)

    def test_ignore_sigusr1(self):
        """
        SIGUSR1 is ignored if we so request.
        """
        # Instantiating LandscapeService should not register the
        # handler if we request to ignore it.
        self.config.ignore_sigusr1 = True
        TestService(self.config)

        handler = signal.getsignal(signal.SIGUSR1)
        self.assertFalse(handler)



from twisted.python.reflect import namedClass

from landscape.service import LandscapeService, run_landscape_service
from landscape.manager.config import ManagerConfiguration
from landscape.broker.amp import RemoteBrokerConnector
from landscape.amp import ComponentPublisher
from landscape.manager.manager import Manager


class ManagerService(LandscapeService):
    """
    The core Twisted Service which creates and runs all necessary managing
    components when started.
    """

    service_name = Manager.name

    def __init__(self, config):
        super(ManagerService, self).__init__(config)
        self.plugins = self.get_plugins()
        self.manager = Manager(self.reactor, self.config)
        self.publisher = ComponentPublisher(self.manager, self.reactor,
                                            self.config)

    def get_plugins(self):
        """Return instances of all the plugins enabled in the configuration."""
        return [namedClass("landscape.manager.%s.%s"
                           % (plugin_name.lower(), plugin_name))()
                for plugin_name in self.config.plugin_factories]

    def startService(self):
        """Start the manager service.

        This method does 3 things, in this order:

          - Start listening for connections on the manager socket.
          - Connect to the broker.
          - Add all configured plugins, that will in turn register themselves.
        """
        super(ManagerService, self).startService()
        self.publisher.start()

        def start_plugins(broker):
            self.broker = broker
            self.manager.broker = broker
            for plugin in self.plugins:
                self.manager.add(plugin)
            return self.broker.register_client(self.service_name)

        self.connector = RemoteBrokerConnector(self.reactor, self.config)
        connected = self.connector.connect()
        return connected.addCallback(start_plugins)

    def stopService(self):
        """Stop the manager and close the connection with the broker."""
        self.connector.disconnect()
        self.publisher.stop()
        super(ManagerService, self).stopService()


def run(args):
    run_landscape_service(ManagerConfiguration, ManagerService, args)



import os
import logging

from landscape.lib.fs import read_file
from landscape.monitor.plugin import MonitorPlugin


REBOOT_REQUIRED_FILENAME = "/var/run/reboot-required"


class RebootRequired(MonitorPlugin):
    """
    Report whether the system requires a reboot.

    @param reboot_required_filename: The path to the flag file that indicates
        if the system needs to be rebooted.
    """

    persist_name = "reboot-required"
    scope = "package"
    run_interval = 900  # 15 minutes
    run_immediately = True

    def __init__(self, reboot_required_filename=REBOOT_REQUIRED_FILENAME):
        self._flag_filename = reboot_required_filename
        self._packages_filename = reboot_required_filename + ".pkgs"

    def _get_flag(self):
        """Return a boolean indicating whether the computer needs a reboot."""
        return os.path.exists(self._flag_filename)

    def _get_packages(self):
        """Return the list of packages that required a reboot, if any."""
        if not os.path.exists(self._packages_filename):
            return []

        lines = read_file(self._packages_filename).splitlines()
        packages = set(line.strip().decode("utf-8") for line in lines if line)
        return sorted(packages)

    def _create_message(self):
        """Return the body of the reboot-required message to be sent."""
        message = {}
        flag = self._get_flag()
        packages = self._get_packages()
        for key, value in [("flag", flag), ("packages", packages)]:
            if value == self._persist.get(key):
                continue
            self._persist.set(key, value)
            message[key] = value
        return message

    def send_message(self):
        """Send a reboot-required message if needed.

        A message will be sent only if the reboot-required status of the
        system has changed.
        """
        message = self._create_message()
        if message:
            message["type"] = "reboot-required-info"
            logging.info("Queueing message with updated "
                         "reboot-required status.")
            self.registry.broker.send_message(message, self._session_id,
                                              urgent=True)

    def run(self):
        """Send reboot-required messages if the server accepts them."""
        return self.registry.broker.call_if_accepted(
            "reboot-required-info", self.send_message)



from landscape.lib.monitor import (
    Timer, Monitor, BurstMonitor, CoverageMonitor, FrequencyMonitor)
from landscape.reactor import FakeReactor
from landscape.tests.helpers import LandscapeTest


class ReactorHavingTest(LandscapeTest):
    def setUp(self):
        super(ReactorHavingTest, self).setUp()
        self.reactor = FakeReactor()


class TimerTest(ReactorHavingTest):

    def setUp(self):
        super(TimerTest, self).setUp()
        self.timer = Timer(create_time=self.reactor.time)

    def test_since_start(self):
        self.assertEqual(self.timer.since_start(), 0.0)

        self.reactor.advance(1)
        self.assertEqual(self.timer.since_start(), 1.0)

        self.reactor.advance(2)
        self.assertEqual(self.timer.since_start(), 3.0)

    def test_since_reset(self):
        self.reactor.advance(1)
        self.assertEqual(self.timer.since_reset(), 1.0)

        self.reactor.advance(1)
        self.assertEqual(self.timer.since_start(), 2.0)

        self.reactor.advance(2)
        self.timer.reset()
        self.assertEqual(self.timer.since_start(), 4.0)


class MonitorTest(ReactorHavingTest):

    def setUp(self):
        super(MonitorTest, self).setUp()
        self.monitor = Monitor("test", create_time=self.reactor.time)

    def test_ping(self):
        self.assertEqual(self.monitor.count, 0)
        self.assertEqual(self.monitor.total_count, 0)

        self.monitor.ping()
        self.assertEqual(self.monitor.count, 1)
        self.assertEqual(self.monitor.total_count, 1)

    def test_reset(self):
        self.assertEqual(self.monitor.count, 0)

        self.monitor.ping()
        self.monitor.ping()
        self.assertEqual(self.monitor.count, 2)
        self.assertEqual(self.monitor.total_count, 2)

        self.monitor.reset()
        self.monitor.ping()
        self.assertEqual(self.monitor.count, 1)
        self.assertEqual(self.monitor.total_count, 3)

    def test_log(self):
        for i in range(100):
            self.monitor.ping()
            self.reactor.advance(1)
        self.monitor.log()
        self.assertTrue("INFO: 100 test events occurred in the last 100.00s."
                        in self.logfile.getvalue())


class BurstMonitorTest(ReactorHavingTest):

    def setUp(self):
        super(BurstMonitorTest, self).setUp()
        self.monitor = BurstMonitor(60, 1, "test",
                                    create_time=self.reactor.time)

    def test_warn_no_pings(self):
        self.assertFalse(self.monitor.warn())

    def test_warn_below_threshold(self):
        self.monitor.ping()
        self.reactor.advance(61)
        self.assertFalse(self.monitor.warn())

    def test_warn_on_threshold(self):
        self.monitor.ping()
        self.reactor.advance(61)
        self.assertFalse(self.monitor.warn())

    def test_warn_over_threshold(self):
        self.monitor.ping()
        self.reactor.advance(30)
        self.monitor.ping()
        self.assertTrue(self.monitor.warn())

        self.reactor.advance(31)
        self.assertFalse(self.monitor.warn())

    def test_warn_in_first_interval(self):
        self.monitor.ping()
        self.reactor.advance(59)
        self.assertFalse(self.monitor.warn())

    def test_warn_unexpected_burst(self):
        self.monitor.ping()
        self.reactor.advance(5000)
        self.assertFalse(self.monitor.warn())

        self.monitor.ping()
        self.assertFalse(self.monitor.warn())

        self.monitor.ping()
        self.assertTrue(self.monitor.warn())

    def test_warn_maximum_count(self):
        monitor = BurstMonitor(60, 2, "test", create_time=self.reactor.time)
        monitor.ping()
        monitor.ping()
        self.assertFalse(monitor.warn())

        monitor.ping()
        self.assertTrue(monitor.warn())

    def test_warn_maximum_count_over_time_span(self):
        monitor = BurstMonitor(60, 3, "test", create_time=self.reactor.time)
        monitor.ping()
        monitor.ping()
        self.assertFalse(monitor.warn())

        self.reactor.advance(30)
        monitor.ping()
        self.assertFalse(monitor.warn())

        self.reactor.advance(31)
        monitor.ping()
        self.assertFalse(monitor.warn())

        monitor.ping()
        monitor.ping()
        self.assertTrue(monitor.warn())


class CoverageMonitorTest(ReactorHavingTest):

    def setUp(self):
        super(CoverageMonitorTest, self).setUp()
        self.monitor = CoverageMonitor(1, 1.0, "test",
                                       create_time=self.reactor.time)

    def test_warn(self):
        self.monitor.ping()
        self.reactor.advance(1)
        self.assertFalse(self.monitor.warn())

        self.reactor.advance(1)
        self.assertTrue(self.monitor.warn())

        self.monitor.reset()
        self.assertFalse(self.monitor.warn())

    def test_percent_no_data(self):
        """
        If no time has passed and the monitor hasn't received any
        pings it should return 100%.
        """
        self.assertEqual(self.monitor.percent, 1.0)

    def test_percent(self):
        self.reactor.advance(1)
        self.assertEqual(self.monitor.percent, 0.0)

        self.monitor.ping()
        self.reactor.advance(1)
        self.assertEqual(self.monitor.percent, 0.5)

    def test_percent_reset(self):
        self.reactor.advance(1)
        self.assertEqual(self.monitor.percent, 0.0)

        self.monitor.reset()
        self.monitor.ping()
        self.reactor.advance(1)
        self.assertEqual(self.monitor.percent, 1.0)

    def test_expected_count(self):
        self.reactor.advance(1)
        self.assertEqual(self.monitor.expected_count, 1.0)

        self.reactor.advance(1)
        self.assertEqual(self.monitor.expected_count, 2.0)

    def test_expected_count_reset(self):
        self.reactor.advance(1)
        self.assertEqual(self.monitor.expected_count, 1.0)

        self.monitor.reset()
        self.reactor.advance(1)
        self.assertEqual(self.monitor.expected_count, 1.0)

    def test_log(self):
        for i in range(100):
            self.monitor.ping()
            self.reactor.advance(1)
        self.monitor.log()
        self.assertTrue("INFO: 100 of 100 expected test events (100.00%) "
                        "occurred in the last 100.00s."
                        in self.logfile.getvalue())

    def test_log_warning(self):
        for i in range(100):
            self.reactor.advance(1)
        self.monitor.log()
        self.assertTrue("WARNING: 0 of 100 expected test events (0.00%) "
                        "occurred in the last 100.00s."
                        in self.logfile.getvalue())


class FrequencyMonitorTest(ReactorHavingTest):

    def setUp(self):
        super(FrequencyMonitorTest, self).setUp()
        self.monitor = FrequencyMonitor(100, 1, "test",
                                        create_time=self.reactor.time)

    def test_expected_count(self):
        self.assertEqual(self.monitor.expected_count, 0)

        self.reactor.advance(99)
        self.assertEqual(self.monitor.expected_count, 0)

        self.reactor.advance(1)
        self.assertEqual(self.monitor.expected_count, 1)

    def test_ping(self):
        self.assertFalse(self.monitor.warn())

        self.reactor.advance(80)
        self.monitor.ping()
        self.assertFalse(self.monitor.warn())

        self.reactor.advance(80)
        self.assertFalse(self.monitor.warn())

    def test_warn(self):
        self.assertFalse(self.monitor.warn())

        self.reactor.advance(101)
        self.assertTrue(self.monitor.warn())

    def test_log(self):
        self.monitor.ping()
        self.reactor.advance(100)
        self.monitor.log()
        self.assertTrue("minimum expected test events"
                        not in self.logfile.getvalue())
        self.reactor.advance(1)
        self.monitor.log()
        self.assertTrue("WARNING: Only 0 of 1 minimum expected test events "
                        "occurred in the last 100.00s."
                        in self.logfile.getvalue())



import sys

from gettext import gettext as _

from gi.repository import Gio, Gtk, Notify

from landscape.ui.model.configuration.proxy import ConfigurationProxy
from landscape.ui.model.configuration.state import ConfigurationModel
from landscape.ui.model.configuration.uisettings import UISettings
from landscape.ui.view.configuration import ClientSettingsDialog
from landscape.ui.controller.configuration import ConfigController


APPLICATION_ID = "com.canonical.landscape-client.settings.ui"
NOTIFY_ID = "Landscape management service"


class SettingsApplicationController(Gtk.Application):
    """
    Core application controller for the landscape settings application.
    """

    def __init__(self, args=[]):
        super(SettingsApplicationController, self).__init__(
            application_id=APPLICATION_ID)
        self._args = args
        self.connect("activate", self.setup_ui)

    def get_config(self):
        return ConfigurationProxy()

    def get_uisettings(self):
        return UISettings(Gio.Settings)

    def on_notify(self, message):
        notification = Notify.Notification.new(NOTIFY_ID, message,
                                               "dialog-information")
        notification.show()

    def on_error(self, message):
        notification = Notify.Notification.new(NOTIFY_ID, message,
                                               "dialog-information")
        notification.show()

    def on_succeed(self, action=None):
        if action:
            message = action
        else:
            message = _("Success.")
        notification = Notify.Notification.new(NOTIFY_ID, message,
                                               "dialog-information")
        notification.show()

    def on_fail(self, action=None):
        if action:
            message = action
        else:
            message = _("Failure.")
        notification = Notify.Notification.new(NOTIFY_ID, message,
                                               "dialog-information")
        notification.show()

    def setup_ui(self, data=None, asynchronous=True):
        """
        L{setup_ui} wires the model to the L{ConfigurationController} and then
        invokes the view with the controller.  When the dialog exits
        appropriate termination is triggered.

        @param data: the Gtk callback could pass this, but it is always None in
        practice.
        @param asynchronous: a parameter passed through to
        L{ConfigurationController.exit}, it indicates whether the exit method
        should be called asynchronously.  Is makes testing easier to use it
        synchronously.
        """
        Notify.init(NOTIFY_ID)
        config = self.get_config()
        uisettings = self.get_uisettings()
        model = ConfigurationModel(proxy=config, proxy_loadargs=self._args,
                                   uisettings=uisettings)
        controller = ConfigController(model)
        if controller.load():
            self.settings_dialog = ClientSettingsDialog(controller)
            if self.settings_dialog.run() == Gtk.ResponseType.OK:
                controller.persist(self.on_notify, self.on_error,
                                   self.on_succeed, self.on_fail)
            controller.exit(asynchronous=asynchronous)
            self.settings_dialog.destroy()
        else:
            self.on_fail(action=_("Authentication failed"))
            sys.stderr.write("Authentication failed.\n")



import subprocess

from landscape.diff import diff
from landscape.lib.process import ProcessInformation
from landscape.lib.jiffies import detect_jiffies
from landscape.monitor.plugin import DataWatcher


class ActiveProcessInfo(DataWatcher):

    message_type = "active-process-info"
    scope = "process"

    def __init__(self, proc_dir="/proc", boot_time=None, jiffies=None,
                 uptime=None, popen=subprocess.Popen):
        super(ActiveProcessInfo, self).__init__()
        self._proc_dir = proc_dir
        self._persist_processes = {}
        self._previous_processes = {}
        self._jiffies_per_sec = jiffies or detect_jiffies()
        self._popen = popen
        self._first_run = True
        self._process_info = ProcessInformation(proc_dir=proc_dir,
                                                jiffies=jiffies,
                                                boot_time=boot_time,
                                                uptime=uptime)

    def register(self, manager):
        super(ActiveProcessInfo, self).register(manager)
        self.call_on_accepted(self.message_type, self.exchange, True)

    def _reset(self):
        """Reset active process data."""
        self._first_run = True
        self._persist_processes = {}
        self._previous_processes = {}

    def get_message(self):
        message = {}
        if self._first_run:
            message["kill-all-processes"] = True
        message.update(self._detect_process_changes())

        if message:
            message["type"] = "active-process-info"
            return message
        return None

    def persist_data(self):
        self._first_run = False
        self._persist_processes = self._previous_processes
        self._previous_processes = {}
        # This forces the registry to write the persistent store to disk
        # This means that the persistent data reflects the state of the
        # messages sent.
        self.registry.flush()

    def _get_processes(self):
        processes = {}
        for process_info in self._process_info.get_all_process_info():
            if process_info["state"] != "X":
                processes[process_info["pid"]] = process_info
        return processes

    def _detect_process_changes(self):
        changes = {}
        processes = self._get_processes()
        creates, updates, deletes = diff(self._persist_processes, processes)
        if creates:
            changes["add-processes"] = list(creates.itervalues())
        if updates:
            changes["update-processes"] = list(updates.itervalues())
        if deletes:
            changes["kill-processes"] = list(deletes.iterkeys())

        # Update cached values for use on the next run.
        self._previous_processes = processes
        return changes



import json
import logging
import socket

from landscape.broker.registration import (
    InvalidCredentialsError, Identity)
from landscape.tests.helpers import LandscapeTest
from landscape.broker.tests.helpers import (
    BrokerConfigurationHelper, RegistrationHelper)
from landscape.lib.persist import Persist
from landscape.lib.vm_info import get_vm_info


class IdentityTest(LandscapeTest):

    helpers = [BrokerConfigurationHelper]

    def setUp(self):
        super(IdentityTest, self).setUp()
        self.persist = Persist(filename=self.makePersistFile())
        self.identity = Identity(self.config, self.persist)

    def check_persist_property(self, attr, persist_name):
        value = "VALUE"
        self.assertEqual(getattr(self.identity, attr), None,
                         "%r attribute should default to None, not %r" %
                         (attr, getattr(self.identity, attr)))
        setattr(self.identity, attr, value)
        self.assertEqual(getattr(self.identity, attr), value,
                         "%r attribute should be %r, not %r" %
                         (attr, value, getattr(self.identity, attr)))
        self.assertEqual(
            self.persist.get(persist_name), value,
            "%r not set to %r in persist" % (persist_name, value))

    def check_config_property(self, attr):
        value = "VALUE"
        setattr(self.config, attr, value)
        self.assertEqual(getattr(self.identity, attr), value,
                         "%r attribute should be %r, not %r" %
                         (attr, value, getattr(self.identity, attr)))

    def test_secure_id(self):
        self.check_persist_property("secure_id",
                                    "registration.secure-id")

    def test_insecure_id(self):
        self.check_persist_property("insecure_id",
                                    "registration.insecure-id")

    def test_computer_title(self):
        self.check_config_property("computer_title")

    def test_account_name(self):
        self.check_config_property("account_name")

    def test_registration_key(self):
        self.check_config_property("registration_key")

    def test_client_tags(self):
        self.check_config_property("tags")

    def test_access_group(self):
        self.check_config_property("access_group")


class RegistrationHandlerTestBase(LandscapeTest):

    helpers = [RegistrationHelper]

    def setUp(self):
        super(RegistrationHandlerTestBase, self).setUp()
        logging.getLogger().setLevel(logging.INFO)
        self.hostname = "ooga.local"
        self.addCleanup(setattr, socket, "getfqdn", socket.getfqdn)
        socket.getfqdn = lambda: self.hostname


class RegistrationHandlerTest(RegistrationHandlerTestBase):

    def test_server_initiated_id_changing(self):
        """
        The server must be able to ask a client to change its secure
        and insecure ids even if no requests were sent.
        """
        self.exchanger.handle_message(
            {"type": "set-id", "id": "abc", "insecure-id": "def"})
        self.assertEqual(self.identity.secure_id, "abc")
        self.assertEqual(self.identity.insecure_id, "def")

    def test_registration_done_event(self):
        """
        When new ids are received from the server, a "registration-done"
        event is fired.
        """
        reactor_mock = self.mocker.patch(self.reactor)
        reactor_mock.fire("registration-done")
        self.mocker.replay()
        self.exchanger.handle_message(
            {"type": "set-id", "id": "abc", "insecure-id": "def"})

    def test_unknown_id(self):
        self.identity.secure_id = "old_id"
        self.identity.insecure_id = "old_id"
        self.mstore.set_accepted_types(["register"])
        self.exchanger.handle_message({"type": "unknown-id"})
        self.assertEqual(self.identity.secure_id, None)
        self.assertEqual(self.identity.insecure_id, None)

    def test_unknown_id_with_clone(self):
        """
        If the server reports us that we are a clone of another computer, then
        set our computer's title accordingly.
        """
        self.config.computer_title = "Wu"
        self.mstore.set_accepted_types(["register"])
        self.exchanger.handle_message({"type": "unknown-id", "clone-of": "Wu"})
        self.assertEqual("Wu (clone)", self.config.computer_title)
        self.assertIn("Client is clone of computer Wu",
                      self.logfile.getvalue())

    def test_should_register(self):
        self.mstore.set_accepted_types(["register"])
        self.config.computer_title = "Computer Title"
        self.config.account_name = "account_name"
        self.assertTrue(self.handler.should_register())

    def test_should_register_with_existing_id(self):
        self.mstore.set_accepted_types(["register"])
        self.identity.secure_id = "secure"
        self.config.computer_title = "Computer Title"
        self.config.account_name = "account_name"
        self.assertFalse(self.handler.should_register())

    def test_should_register_without_computer_title(self):
        self.mstore.set_accepted_types(["register"])
        self.config.computer_title = None
        self.assertFalse(self.handler.should_register())

    def test_should_register_without_account_name(self):
        self.mstore.set_accepted_types(["register"])
        self.config.account_name = None
        self.assertFalse(self.handler.should_register())

    def test_should_register_with_unaccepted_message(self):
        self.assertFalse(self.handler.should_register())

    def test_queue_message_on_exchange(self):
        """
        When a computer_title and account_name are available, no
        secure_id is set, and an exchange is about to happen,
        queue a registration message.
        """
        self.mstore.set_accepted_types(["register"])
        self.config.computer_title = "Computer Title"
        self.config.account_name = "account_name"
        self.reactor.fire("pre-exchange")
        messages = self.mstore.get_pending_messages()
        self.assertEqual(1, len(messages))
        self.assertEqual("register", messages[0]["type"])
        self.assertEqual(self.logfile.getvalue().strip(),
                         "INFO: Queueing message to register with account "
                         "'account_name' without a password.")

    def test_queue_message_on_exchange_with_vm_info(self):
        """
        When a computer_title and account_name are available, no
        secure_id is set, and an exchange is about to happen,
        queue a registration message with VM information.
        """
        get_vm_info_mock = self.mocker.replace(get_vm_info)
        get_vm_info_mock()
        self.mocker.result("vmware")
        self.mocker.replay()
        self.mstore.set_accepted_types(["register"])
        self.config.computer_title = "Computer Title"
        self.config.account_name = "account_name"
        self.reactor.fire("pre-exchange")
        messages = self.mstore.get_pending_messages()
        self.assertEqual("vmware", messages[0]["vm-info"])
        self.assertEqual(self.logfile.getvalue().strip(),
                         "INFO: Queueing message to register with account "
                         "'account_name' without a password.")

    def test_queue_message_on_exchange_with_lxc_container(self):
        """
        If the client is running in an LXC container, the information is
        included in the registration message.
        """
        get_container_info_mock = self.mocker.replace(
            "landscape.lib.vm_info.get_container_info")
        get_container_info_mock()
        self.mocker.result("lxc")
        self.mocker.replay()
        self.mstore.set_accepted_types(["register"])
        self.config.computer_title = "Computer Title"
        self.config.account_name = "account_name"
        self.reactor.fire("pre-exchange")
        messages = self.mstore.get_pending_messages()
        self.assertEqual("lxc", messages[0]["container-info"])

    def test_queue_message_on_exchange_with_password(self):
        """If a registration password is available, we pass it on!"""
        self.mstore.set_accepted_types(["register"])
        self.config.computer_title = "Computer Title"
        self.config.account_name = "account_name"
        self.config.registration_key = "SEKRET"
        self.reactor.fire("pre-exchange")
        messages = self.mstore.get_pending_messages()
        password = messages[0]["registration_password"]
        self.assertEqual("SEKRET", password)
        self.assertEqual(self.logfile.getvalue().strip(),
                         "INFO: Queueing message to register with account "
                         "'account_name' with a password.")

    def test_queue_message_on_exchange_with_tags(self):
        """
        If the admin has defined tags for this computer, we send them to the
        server.
        """
        self.mstore.set_accepted_types(["register"])
        self.config.computer_title = "Computer Title"
        self.config.account_name = "account_name"
        self.config.registration_key = "SEKRET"
        self.config.tags = u"computer,tag"
        self.reactor.fire("pre-exchange")
        messages = self.mstore.get_pending_messages()
        self.assertEqual("computer,tag", messages[0]["tags"])
        self.assertEqual(self.logfile.getvalue().strip(),
                         "INFO: Queueing message to register with account "
                         "'account_name' and tags computer,tag with a "
                         "password.")

    def test_queue_message_on_exchange_with_invalid_tags(self):
        """
        If the admin has defined tags for this computer, but they are not
        valid, we drop them, and report an error.
        """
        self.log_helper.ignore_errors("Invalid tags provided for registration")
        self.mstore.set_accepted_types(["register"])
        self.config.computer_title = "Computer Title"
        self.config.account_name = "account_name"
        self.config.registration_key = "SEKRET"
        self.config.tags = u"<script>alert()</script>"
        self.reactor.fire("pre-exchange")
        messages = self.mstore.get_pending_messages()
        self.assertIs(None, messages[0]["tags"])
        self.assertEqual(self.logfile.getvalue().strip(),
                         "ERROR: Invalid tags provided for registration.\n    "
                         "INFO: Queueing message to register with account "
                         "'account_name' with a password.")

    def test_queue_message_on_exchange_with_unicode_tags(self):
        """
        If the admin has defined tags for this computer, we send them to the
        server.
        """
        self.mstore.set_accepted_types(["register"])
        self.config.computer_title = "Computer Title"
        self.config.account_name = "account_name"
        self.config.registration_key = "SEKRET"
        self.config.tags = u"prova\N{LATIN SMALL LETTER J WITH CIRCUMFLEX}o"
        self.reactor.fire("pre-exchange")
        messages = self.mstore.get_pending_messages()
        expected = u"prova\N{LATIN SMALL LETTER J WITH CIRCUMFLEX}o"
        self.assertEqual(expected, messages[0]["tags"])

        self.assertEqual(self.logfile.getvalue().strip(),
                         "INFO: Queueing message to register with account "
                         "'account_name' and tags prova\xc4\xb5o "
                         "with a password.")

    def test_queue_message_on_exchange_with_access_group(self):
        """
        If the admin has defined an access_group for this computer, we send
        it to the server.
        """
        self.mstore.set_accepted_types(["register"])
        self.config.account_name = "account_name"
        self.config.access_group = u"dinosaurs"
        self.config.tags = u"server,london"
        self.reactor.fire("pre-exchange")
        messages = self.mstore.get_pending_messages()
        self.assertEqual("dinosaurs", messages[0]["access_group"])
        self.assertEqual(self.logfile.getvalue().strip(),
                         "INFO: Queueing message to register with account "
                         "'account_name' in access group 'dinosaurs' and "
                         "tags server,london without a password.")

    def test_queue_message_on_exchange_with_empty_access_group(self):
        """
        If the access_group is "", then the outgoing message does not define
        an "access_group" key.
        """
        self.mstore.set_accepted_types(["register"])
        self.config.access_group = u""
        self.reactor.fire("pre-exchange")
        messages = self.mstore.get_pending_messages()
        # Make sure the key does not appear in the outgoing message.
        self.assertNotIn("access_group", messages[0])

    def test_queue_message_on_exchange_with_none_access_group(self):
        """
        If the access_group is None, then the outgoing message does not define
        an "access_group" key.
        """
        self.mstore.set_accepted_types(["register"])
        self.config.access_group = None
        self.reactor.fire("pre-exchange")
        messages = self.mstore.get_pending_messages()
        # Make sure the key does not appear in the outgoing message.
        self.assertNotIn("access_group", messages[0])

    def test_queueing_registration_message_resets_message_store(self):
        """
        When a registration message is queued, the store is reset
        entirely, since everything else that was queued is meaningless
        now that we're trying to register again.
        """
        self.mstore.set_accepted_types(["register", "test"])
        self.mstore.add({"type": "test"})
        self.config.computer_title = "Computer Title"
        self.config.account_name = "account_name"
        self.reactor.fire("pre-exchange")
        messages = self.mstore.get_pending_messages()
        self.assertEqual(len(messages), 1)
        self.assertEqual(messages[0]["type"], "register")

    def test_no_message_when_should_register_is_false(self):
        """If we already have a secure id, do not queue a register message.
        """
        handler_mock = self.mocker.patch(self.handler)
        handler_mock.should_register()
        self.mocker.result(False)

        self.mstore.set_accepted_types(["register"])
        self.config.computer_title = "Computer Title"
        self.config.account_name = "account_name"

        # If we didn't fake it, it'd work.  We do that to ensure that
        # all the needed data is in place, and that this method is
        # really what decides if a message is sent or not.  This way
        # we can test it individually.
        self.assertTrue(self.handler.should_register())

        # Now let's see.
        self.mocker.replay()

        self.reactor.fire("pre-exchange")
        self.assertMessages(self.mstore.get_pending_messages(), [])

    def test_registration_failed_event(self):
        """
        The deferred returned by a registration request should fail
        with L{InvalidCredentialsError} if the server responds with a
        failure message.
        """
        reactor_mock = self.mocker.patch(self.reactor)
        reactor_mock.fire("registration-failed")
        self.mocker.replay()
        self.exchanger.handle_message(
            {"type": "registration", "info": "unknown-account"})

    def test_registration_failed_event_not_fired_when_uncertain(self):
        """
        If the data in the registration message isn't what we expect,
        the event isn't fired.
        """
        reactor_mock = self.mocker.patch(self.reactor)
        reactor_mock.fire("registration-failed")
        self.mocker.count(0)
        self.mocker.replay()
        self.exchanger.handle_message(
            {"type": "registration", "info": "blah-blah"})

    def test_register_resets_ids(self):
        self.identity.secure_id = "foo"
        self.identity.insecure_id = "bar"
        self.handler.register()
        self.assertEqual(self.identity.secure_id, None)
        self.assertEqual(self.identity.insecure_id, None)

    def test_register_calls_urgent_exchange(self):
        exchanger_mock = self.mocker.patch(self.exchanger)
        exchanger_mock.exchange()
        self.mocker.passthrough()
        self.mocker.replay()
        self.handler.register()

    def test_register_deferred_called_on_done(self):
        # We don't want informational messages.
        self.logger.setLevel(logging.WARNING)

        calls = [0]
        d = self.handler.register()

        def add_call(result):
            self.assertEqual(result, None)
            calls[0] += 1

        d.addCallback(add_call)

        # This should somehow callback the deferred.
        self.exchanger.handle_message(
            {"type": "set-id", "id": "abc", "insecure-id": "def"})

        self.assertEqual(calls, [1])

        # Doing it again to ensure that the deferred isn't called twice.
        self.exchanger.handle_message(
            {"type": "set-id", "id": "abc", "insecure-id": "def"})

        self.assertEqual(calls, [1])

        self.assertEqual(self.logfile.getvalue(), "")

    def test_resynchronize_fired_when_registration_done(self):
        """
        When we call C{register} this should trigger a "resynchronize-clients"
        event with global scope.
        """
        results = []

        def append(scopes=None):
            results.append(scopes)

        self.reactor.call_on("resynchronize-clients", append)

        self.handler.register()

        # This should somehow callback the deferred.
        self.exchanger.handle_message(
            {"type": "set-id", "id": "abc", "insecure-id": "def"})

        self.assertEqual(results, [None])

    def test_register_deferred_called_on_failed(self):
        # We don't want informational messages.
        self.logger.setLevel(logging.WARNING)

        calls = [0]
        d = self.handler.register()

        def add_call(failure):
            exception = failure.value
            self.assertTrue(isinstance(exception, InvalidCredentialsError))
            calls[0] += 1

        d.addErrback(add_call)

        # This should somehow callback the deferred.
        self.exchanger.handle_message(
            {"type": "registration", "info": "unknown-account"})

        self.assertEqual(calls, [1])

        # Doing it again to ensure that the deferred isn't called twice.
        self.exchanger.handle_message(
            {"type": "registration", "info": "unknown-account"})

        self.assertEqual(calls, [1])

        self.assertEqual(self.logfile.getvalue(), "")

    def test_exchange_done_calls_exchange(self):
        exchanger_mock = self.mocker.patch(self.exchanger)
        exchanger_mock.exchange()
        self.mocker.passthrough()
        self.mocker.replay()

        self.mstore.set_accepted_types(["register"])
        self.config.computer_title = "Computer Title"
        self.config.account_name = "account_name"
        self.reactor.fire("exchange-done")

    def test_exchange_done_wont_call_exchange_when_just_tried(self):
        exchanger_mock = self.mocker.patch(self.exchanger)
        exchanger_mock.exchange()
        self.mocker.count(0)
        self.mocker.replay()

        self.mstore.set_accepted_types(["register"])
        self.config.computer_title = "Computer Title"
        self.config.account_name = "account_name"
        self.reactor.fire("pre-exchange")
        self.reactor.fire("exchange-done")

    def test_default_hostname(self):
        self.mstore.set_accepted_types(["register"])
        self.config.computer_title = "Computer Title"
        self.config.account_name = "account_name"
        self.config.registration_key = "SEKRET"
        self.reactor.fire("pre-exchange")
        messages = self.mstore.get_pending_messages()
        self.assertEqual(socket.getfqdn(), messages[0]["hostname"])


class JujuRegistrationHandlerTest(RegistrationHandlerTestBase):

    juju_contents = json.dumps({"environment-uuid": "DEAD-BEEF",
                                "machine-id": "1",
                                "api-addresses": "10.0.3.1:17070"})

    def test_juju_info_added_when_present(self):
        """
        When information about the Juju environment is found in
        the $data_dir/juju-info.d/ directory, it's included in
        the registration message.
        """
        self.mstore.set_accepted_types(["register"])
        self.mstore.set_server_api("3.3")
        self.config.account_name = "account_name"
        self.reactor.fire("run")
        self.reactor.fire("pre-exchange")

        messages = self.mstore.get_pending_messages()
        self.assertEqual(
            {"environment-uuid": "DEAD-BEEF",
             "machine-id": "1",
             "api-addresses": ["10.0.3.1:17070"]},
            messages[0]["juju-info"])

    def test_juju_info_skipped_with_old_server(self):
        """
        If a server doesn't speak at least 3.3, the juju-info field is
        isn't included in the message.
        """
        self.mstore.set_accepted_types(["register"])
        self.mstore.set_server_api("3.2")
        self.config.account_name = "account_name"
        self.reactor.fire("run")
        self.reactor.fire("pre-exchange")

        messages = self.mstore.get_pending_messages()
        self.assertNotIn("juju-info", messages[0])



import socket
from landscape.monitor.networkactivity import NetworkActivity
from landscape.tests.helpers import LandscapeTest, MonitorHelper


class NetworkActivityTest(LandscapeTest):

    helpers = [MonitorHelper]

    stats_template = """\
Inter-|   Receive                           |  Transmit
 face |bytes    packets compressed multicast|bytes    packets errs drop fifo
    lo:%(lo_in)d   %(lo_in_p)d   0     0   %(lo_out)d %(lo_out_p)d  0  0  0
    eth0: %(eth0_in)d   12539    0     62  %(eth0_out)d   12579  0  0  0
    %(extra)s
"""

    def setUp(self):
        super(NetworkActivityTest, self).setUp()
        self.activity_file = open(self.makeFile(), "w+")
        self.write_activity()
        self.plugin = NetworkActivity(
            network_activity_file=self.activity_file.name,
            create_time=self.reactor.time)
        self.monitor.add(self.plugin)

    def tearDown(self):
        self.activity_file.close()
        super(NetworkActivityTest, self).tearDown()

    def write_activity(self, lo_in=0, lo_out=0, eth0_in=0, eth0_out=0,
                        extra="", lo_in_p=0, lo_out_p=0, **kw):
        kw.update(dict(
            lo_in=lo_in,
            lo_out=lo_out,
            lo_in_p=lo_in_p,
            lo_out_p=lo_out_p,
            eth0_in=eth0_in,
            eth0_out=eth0_out,
            extra=extra))
        self.activity_file.seek(0, 0)
        self.activity_file.truncate()
        self.activity_file.write(self.stats_template % kw)
        self.activity_file.flush()

    def test_read_proc_net_dev(self):
        """
        When the network activity plugin runs it reads data from
        /proc/net/dev which it parses and accumulates to read values.
        This test ensures that /proc/net/dev is always parseable and
        that messages are in the expected format and contain data with
        expected datatypes.
        """
        plugin = NetworkActivity(create_time=self.reactor.time)
        self.monitor.add(plugin)
        plugin.run()
        self.reactor.advance(self.monitor.step_size)
        # hmmm. try to connect anywhere to advance the net stats
        try:
            socket.socket().connect(("localhost", 9999))
        except socket.error:
            pass
        plugin.run()
        message = plugin.create_message()
        self.assertTrue(message)

    def test_message_contents(self):
        """
        The network plugin sends messages with the traffic delta along with the
        step per network interface. Only interfaces which have deltas are
        present in the message.
        """
        self.write_activity(lo_in=2000, lo_out=1900)
        self.plugin.run()
        self.reactor.advance(self.monitor.step_size)
        self.write_activity(lo_in=2010, lo_out=1999)
        self.plugin.run()
        message = self.plugin.create_message()
        self.assertTrue(message)
        self.assertTrue("type" in message)
        self.assertEqual(message["type"], "network-activity")
        self.assertEqual(message["activities"]["lo"],
                         [(300, 10, 99)])
        self.assertNotIn("eth0", message["activities"])

    def test_proc_rollover(self):
        """
        If /proc/net/dev rollovers, the network plugin handles the value and
        gives a positive value instead.
        """
        self.plugin._rollover_maxint = 10000
        self.write_activity(lo_in=2000, lo_out=1900)
        self.plugin.run()
        self.reactor.advance(self.monitor.step_size)
        self.write_activity(lo_in=1010, lo_out=999)
        self.plugin.run()
        message = self.plugin.create_message()
        self.assertTrue(message)
        self.assertTrue("type" in message)
        self.assertEqual(message["type"], "network-activity")
        self.assertEqual(message["activities"]["lo"],
                         [(300, 9010, 9099)])
        self.assertNotIn("eth0", message["activities"])

    def test_no_message_without_traffic_delta(self):
        """
        If no traffic delta is detected between runs, no message will be
        generated by the plugin.
        """
        self.plugin.run()
        self.reactor.advance(self.monitor.step_size)
        message = self.plugin.create_message()
        self.assertFalse(message)
        self.plugin.run()
        message = self.plugin.create_message()
        self.assertFalse(message)

    def test_no_message_without_traffic_delta_across_steps(self):
        """
        A traffic delta needs to cross step boundaries before a message
        is generated.
        """
        self.plugin.run()
        self.write_activity(lo_out=1000, eth0_out=1000)
        self.reactor.advance(self.monitor.step_size)
        message = self.plugin.create_message()
        self.assertFalse(message)

    def test_interface_temporarily_disappears(self):
        """
        When an interface is removed (ie usb hotplug) and then activated again
        its delta will not be retained, because the values may have been reset.
        """
        self.write_activity(extra="wlan0: 2222 0 0 0 2222 0 0 0 0")
        self.plugin.run()
        self.reactor.advance(self.monitor.step_size)
        self.write_activity()
        self.plugin.run()
        message = self.plugin.create_message()
        self.assertFalse(message)
        self.write_activity(extra="wlan0: 1000 0 0 0 1000 0 0 0 0")
        self.reactor.advance(self.monitor.step_size)
        self.plugin.run()
        message = self.plugin.create_message()
        self.assertFalse(message)

    def test_messaging_flushes(self):
        """
        Duplicate message should never be created.  If no data is available, no
        message is created.
        """
        self.plugin.run()
        self.reactor.advance(self.monitor.step_size)
        self.write_activity(eth0_out=1111)
        self.plugin.run()
        message = self.plugin.create_message()
        self.assertTrue(message)
        message = self.plugin.create_message()
        self.assertFalse(message)

    def test_exchange_no_message(self):
        """
        No message is sent to the exchange if there isn't a traffic delta.
        """
        self.reactor.advance(self.monitor.step_size)
        self.mstore.set_accepted_types([self.plugin.message_type])
        self.plugin.exchange()
        self.assertFalse(self.mstore.count_pending_messages())

    def test_exchange_messages(self):
        """
        The network plugin queues message when an exchange happens. Each
        message should be aligned to a step boundary; messages collected
        between exchange periods should be delivered in a single message.
        """
        self.reactor.advance(self.monitor.step_size)
        self.write_activity(lo_out=1000, eth0_out=1000)
        self.plugin.run()
        self.mstore.set_accepted_types([self.plugin.message_type])
        self.plugin.exchange()
        step_size = self.monitor.step_size
        self.assertMessages(self.mstore.get_pending_messages(),
                        [{"type": "network-activity",
                          "activities": {
                              "lo": [(step_size, 0, 1000)],
                              "eth0": [(step_size, 0, 1000)]}}])

    def test_config(self):
        """The network activity plugin is enabled by default."""
        self.assertIn("NetworkActivity", self.config.plugin_factories)

    def test_limit_amount_of_items(self):
        """
        The network plugin doesn't send too many items at once in a single
        network message, to not crush the server.
        """
        def extra(data):
            result = ""
            for i in range(50):
                result += (
"""eth%d: %d   12539      0     62  %d   12579    0    0   0\n    """
                    % (i, data, data))
            return result
        for i in range(1, 10):
            data = i * 1000
            self.write_activity(lo_out=data, eth0_out=data, extra=extra(data))
            self.plugin.run()
            self.reactor.advance(self.monitor.step_size)
        # We have created 408 items. It should be sent in 3 messages.
        message = self.plugin.create_message()
        items = sum(len(i) for i in message["activities"].values())
        self.assertEqual(200, items)
        message = self.plugin.create_message()
        items = sum(len(i) for i in message["activities"].values())
        self.assertEqual(200, items)
        message = self.plugin.create_message()
        items = sum(len(i) for i in message["activities"].values())
        self.assertEqual(8, items)



import tempfile

from twisted.internet.defer import succeed

from landscape.monitor.mountinfo import MountInfo
from landscape.tests.helpers import LandscapeTest, mock_counter, MonitorHelper
from landscape.tests.mocker import ANY


mb = lambda x: x * 1024 * 1024


class MountInfoTest(LandscapeTest):
    """Tests for mount-info plugin."""

    helpers = [MonitorHelper]

    def setUp(self):
        LandscapeTest.setUp(self)
        self.mstore.set_accepted_types(["mount-info", "free-space"])
        self.log_helper.ignore_errors("Typelib file for namespace")

    def get_mount_info(self, *args, **kwargs):
        if "statvfs" not in kwargs:
            kwargs["statvfs"] = lambda path: (0,) * 1000
        plugin = MountInfo(*args, **kwargs)
        # To make sure tests are isolated from the real system by default.
        plugin.is_device_removable = lambda x: False
        return plugin

    def test_read_proc_mounts(self):
        """
        When the mount info plugin runs it reads data from
        /proc/mounts to discover mounts and calls os.statvfs() to
        retrieve current data for each mount point.  This test makes
        sure that os.statvfs() is called without failing, that
        /proc/mounts is readable, and that messages with the expected
        datatypes are generated.
        """
        plugin = self.get_mount_info(create_time=self.reactor.time)
        self.monitor.add(plugin)

        self.reactor.advance(self.monitor.step_size)

        message = plugin.create_mount_info_message()
        self.assertTrue(message)
        self.assertEqual(message["type"], "mount-info")
        self.assertTrue("mount-info" in message)
        self.assertTrue(len(message["mount-info"]) > 0)

        keys = set(["filesystem", "total-space", "device", "mount-point"])
        for now, mount_info in message["mount-info"]:
            self.assertEqual(set(mount_info.keys()), keys)
            self.assertTrue(isinstance(mount_info["filesystem"], basestring))
            self.assertTrue(isinstance(mount_info["device"], basestring))
            self.assertTrue(isinstance(mount_info["total-space"], (int, long)))
            self.assertTrue(isinstance(mount_info["mount-point"], basestring))

    def test_read_sample_data(self):
        """
        Sample data is used to ensure that the free space included in
        the message is calculated correctly.
        """
        def statvfs(path):
            if path == "/":
                return (4096, 0, mb(1000L), mb(100L), 0L, 0L, 0L, 0, 0)
            else:
                return (4096, 0, mb(10000L), mb(1000L), 0L, 0L, 0L, 0, 0)

        filename = self.makeFile("""\
rootfs / rootfs rw 0 0
none /dev ramfs rw 0 0
/dev/hda1 / ext3 rw 0 0
/dev/hda1 /dev/.static/dev ext3 rw 0 0
proc /proc proc rw,nodiratime 0 0
sysfs /sys sysfs rw 0 0
usbfs /proc/bus/usb usbfs rw 0 0
devpts /dev/pts devpts rw 0 0
tmpfs /dev/shm tmpfs rw 0 0
tmpfs /lib/modules/2.6.12-10-386/volatile tmpfs rw 0 0
/dev/hde1 /mnt/hde1 reiserfs rw 0 0
/dev/hde1 /mnt/bind reiserfs rw 0 0
/dev/sdb2 /media/Boot\\040OSX hfsplus nls=utf8 0 0
""")

        mtab_filename = self.makeFile("""\
rootfs / rootfs rw 0 0
none /dev ramfs rw 0 0
/dev/hda1 / ext3 rw 0 0
/dev/hda1 /dev/.static/dev ext3 rw 0 0
proc /proc proc rw,nodiratime 0 0
sysfs /sys sysfs rw 0 0
usbfs /proc/bus/usb usbfs rw 0 0
devpts /dev/pts devpts rw 0 0
tmpfs /dev/shm tmpfs rw 0 0
tmpfs /lib/modules/2.6.12-10-386/volatile tmpfs rw 0 0
/dev/hde1 /mnt/hde1 reiserfs rw 0 0
/dev/hde1 /mnt/bind none rw,bind 0 0
/dev/sdb2 /media/Boot\\040OSX hfsplus rw 0 0
""")
        plugin = self.get_mount_info(mounts_file=filename, statvfs=statvfs,
                                     create_time=self.reactor.time,
                                     mtab_file=mtab_filename)
        self.monitor.add(plugin)
        self.reactor.advance(self.monitor.step_size)

        message = plugin.create_mount_info_message()
        self.assertTrue(message)
        self.assertEqual(message["type"], "mount-info")

        mount_info = message.get("mount-info", ())

        self.assertEqual(len(mount_info), 3)

        self.assertEqual(mount_info[0][1],
                         {"device": "/dev/hda1", "mount-point": "/",
                          "filesystem": "ext3", "total-space": 4096000})

        self.assertEqual(mount_info[1][1],
                         {"device": "/dev/hde1", "mount-point": "/mnt/hde1",
                          "filesystem": "reiserfs", "total-space": 40960000})

        self.assertEqual(
            mount_info[2][1],
            {"device": "/dev/sdb2", "mount-point": "/media/Boot OSX",
             "filesystem": "hfsplus", "total-space": 40960000})

    def test_read_changing_total_space(self):
        """
        Total space measurements are only sent when (a) none have ever
        been sent, or (b) the value has changed since the last time
        data was collected.  The test sets the mount info plugin
        interval to the same value as the step size and advances the
        reactor such that the plugin will be run twice.  Each time it
        runs it gets a different value from our sample statvfs()
        function which should cause it to queue new messages.
        """
        def statvfs(path, multiplier=mock_counter(1).next):
            return (4096, 0, mb(multiplier() * 1000), mb(100), 0, 0, 0, 0, 0)

        filename = self.makeFile("""\
/dev/hda1 / ext3 rw 0 0
""")
        plugin = self.get_mount_info(mounts_file=filename, statvfs=statvfs,
                                     create_time=self.reactor.time,
                                     interval=self.monitor.step_size,
                                     mtab_file=filename)
        self.monitor.add(plugin)

        self.reactor.advance(self.monitor.step_size * 2)

        message = plugin.create_mount_info_message()
        mount_info = message["mount-info"]
        self.assertEqual(len(mount_info), 2)

        for i, total_space in enumerate([4096000, 8192000]):
            self.assertEqual(mount_info[i][0],
                             (i + 1) * self.monitor.step_size)
            self.assertEqual(mount_info[i][1],
                             {"device": "/dev/hda1", "filesystem": "ext3",
                              "mount-point": "/", "total-space": total_space})

    def test_read_disjointed_changing_total_space(self):
        """
        Total space measurements are only sent when (a) none have ever
        been sent, or (b) the value has changed since the last time
        data was collected.  This test ensures that the (b) criteria
        is checked per-mount point.  The sample statvfs() function
        only provides changing total space for /; therefore, new
        messages should only be queued for / after the first message
        is created.
        """
        def statvfs(path, multiplier=mock_counter(1).next):
            if path == "/":
                return (4096, 0, mb(1000), mb(100), 0, 0, 0, 0, 0)
            return (4096, 0, mb(multiplier() * 1000), mb(100), 0, 0, 0, 0, 0)

        filename = self.makeFile("""\
/dev/hda1 / ext3 rw 0 0
/dev/hde1 /mnt/hde1 ext3 rw 0 0
""")
        plugin = self.get_mount_info(mounts_file=filename, statvfs=statvfs,
                                     create_time=self.reactor.time,
                                     interval=self.monitor.step_size,
                                     mtab_file=filename)
        self.monitor.add(plugin)

        self.reactor.advance(self.monitor.step_size * 2)

        message = plugin.create_mount_info_message()
        self.assertTrue(message)

        mount_info = message.get("mount-info", ())
        self.assertEqual(len(mount_info), 3)

        self.assertEqual(mount_info[0][0], self.monitor.step_size)
        self.assertEqual(mount_info[0][1],
                         {"device": "/dev/hda1", "mount-point": "/",
                          "filesystem": "ext3", "total-space": 4096000})

        self.assertEqual(mount_info[1][0], self.monitor.step_size)
        self.assertEqual(mount_info[1][1],
                         {"device": "/dev/hde1", "mount-point": "/mnt/hde1",
                          "filesystem": "ext3", "total-space": 4096000})

        self.assertEqual(mount_info[2][0], self.monitor.step_size * 2)
        self.assertEqual(mount_info[2][1],
                         {"device": "/dev/hde1", "mount-point": "/mnt/hde1",
                          "filesystem": "ext3", "total-space": 8192000})

    def test_exchange_messages(self):
        """
        The mount_info plugin queues message when manager.exchange()
        is called.  Each message should be aligned to a step boundary;
        messages collected bewteen exchange periods should be
        delivered in a single message.
        """
        def statvfs(path):
            return (4096, 0, mb(1000), mb(100), 0, 0, 0, 0, 0)

        filename = self.makeFile("""\
/dev/hda1 / ext3 rw 0 0
""")
        plugin = self.get_mount_info(mounts_file=filename, statvfs=statvfs,
                                     create_time=self.reactor.time,
                                     mtab_file=filename)
        step_size = self.monitor.step_size
        self.monitor.add(plugin)

        # Exchange should trigger a flush of the persist database
        registry_mocker = self.mocker.replace(plugin.registry)
        registry_mocker.flush()
        self.mocker.result(None)
        self.mocker.replay()

        self.reactor.advance(step_size * 2)
        self.monitor.exchange()

        messages = self.mstore.get_pending_messages()
        self.assertEqual(len(messages), 2)

        message = [d for d in messages if d["type"] == "free-space"][0]
        free_space = message["free-space"]
        for i in range(len(free_space)):
            self.assertEqual(free_space[i][0], (i + 1) * step_size)
            self.assertEqual(free_space[i][1], "/")
            self.assertEqual(free_space[i][2], 409600)

    def test_messaging_flushes(self):
        """
        Duplicate message should never be created.  If no data is
        available, None will be returned when messages are created.
        """
        def statvfs(path):
            return (4096, 0, mb(1000), mb(100), 0, 0, 0, 0, 0)

        filename = self.makeFile("""\
/dev/hda1 / ext3 rw 0 0
""")
        plugin = self.get_mount_info(mounts_file=filename, statvfs=statvfs,
                                     create_time=self.reactor.time,
                                     mtab_file=filename)
        self.monitor.add(plugin)

        self.reactor.advance(self.monitor.step_size)

        messages = plugin.create_messages()
        self.assertEqual(len(messages), 2)

        messages = plugin.create_messages()
        self.assertEqual(len(messages), 0)

    def test_read_multi_bound_mounts(self):
        """
        The mount info plugin should handle multi-bound mount points
        by reporting them only once.  In practice, this test doesn't
        really test anything since the current behaviour is to ignore
        any mount point for which the device doesn't start with /dev.
        """
        def statvfs(path):
            return (4096, 0, mb(1000), mb(100), 0, 0, 0, 0, 0)

        filename = self.makeFile("""\
/dev/hdc4 /mm xfs rw 0 0
/mm/ubuntu-mirror /home/dchroot/warty/mirror none bind 0 0
/mm/ubuntu-mirror /home/dchroot/hoary/mirror none bind 0 0
/mm/ubuntu-mirror /home/dchroot/breezy/mirror none bind 0 0
""")
        plugin = self.get_mount_info(mounts_file=filename, statvfs=statvfs,
                                     create_time=self.reactor.time,
                                     mtab_file=filename)
        step_size = self.monitor.step_size
        self.monitor.add(plugin)

        self.reactor.advance(step_size)

        message = plugin.create_mount_info_message()
        self.assertTrue(message)

        mount_info = message.get("mount-info", ())
        self.assertEqual(len(mount_info), 1)

        self.assertEqual(mount_info[0][0], step_size)
        self.assertEqual(mount_info[0][1],
                         {"device": "/dev/hdc4", "mount-point": "/mm",
                          "filesystem": "xfs", "total-space": 4096000})

    def test_ignore_nfs_mounts(self):
        """
        The mount info plugin should only report data about local
        mount points.
        """

        filename = self.makeFile("""\
ennui:/data /data nfs rw,v3,rsize=32768,wsize=32768,hard,lock,proto=udp,\
addr=ennui 0 0
""")
        plugin = self.get_mount_info(mounts_file=filename, mtab_file=filename)
        self.monitor.add(plugin)
        plugin.run()

        message = plugin.create_mount_info_message()
        self.assertEqual(message, None)

    def test_ignore_removable_partitions(self):
        """
        "Removable" partitions are not reported to the server.
        """
        filename = self.makeFile("""\
/dev/hdc4 /mm xfs rw 0 0""")
        plugin = self.get_mount_info(mounts_file=filename, mtab_file=filename)

        plugin.is_device_removable = lambda x: True  # They are all removable

        self.monitor.add(plugin)
        plugin.run()
        message = plugin.create_mount_info_message()
        self.assertEqual(message, None)

    def test_sample_free_space(self):
        """Test collecting information about free space."""
        def statvfs(path, multiplier=mock_counter(1).next):
            return (4096, 0, mb(1000), mb(multiplier() * 100), 0, 0, 0, 0, 0)

        filename = self.makeFile("""\
/dev/hda2 / xfs rw 0 0
""")
        plugin = self.get_mount_info(mounts_file=filename, statvfs=statvfs,
                                     create_time=self.reactor.time,
                                     mtab_file=filename)
        step_size = self.monitor.step_size
        self.monitor.add(plugin)

        self.reactor.advance(step_size)

        message = plugin.create_free_space_message()
        self.assertTrue(message)
        self.assertEqual(message.get("type"), "free-space")
        free_space = message.get("free-space", ())
        self.assertEqual(len(free_space), 1)
        self.assertEqual(free_space[0], (step_size, "/", 409600))

    def test_never_exchange_empty_messages(self):
        """
        When the plugin has no data, it's various create_X_message()
        methods will return None.  Empty or null messages should never
        be queued.
        """
        self.mstore.set_accepted_types(["load-average"])

        filename = self.makeFile("")
        plugin = self.get_mount_info(mounts_file=filename, mtab_file=filename)
        self.monitor.add(plugin)
        self.monitor.exchange()
        self.assertEqual(len(self.mstore.get_pending_messages()), 0)

    def test_messages(self):
        """
        Test ensures all expected messages are created and contain the
        right datatypes.
        """
        def statvfs(path):
            return (4096, 0, mb(1000), mb(100), 0, 0, 0, 0, 0)

        filename = self.makeFile("""\
/dev/hda2 / xfs rw 0 0
""")
        plugin = self.get_mount_info(mounts_file=filename, statvfs=statvfs,
                                     create_time=self.reactor.time,
                                     mtab_file=filename)
        step_size = self.monitor.step_size
        self.monitor.add(plugin)

        self.reactor.advance(step_size)
        self.monitor.exchange()

        messages = self.mstore.get_pending_messages()
        self.assertEqual(len(messages), 2)
        self.assertEqual(messages[0].get("mount-info"),
                         [(step_size,
                           {"device": "/dev/hda2", "mount-point": "/",
                            "filesystem": "xfs", "total-space": 4096000})])
        self.assertEqual(messages[1].get("free-space"),
                         [(step_size, "/", 409600)])
        self.assertTrue(isinstance(messages[1]["free-space"][0][2],
                                   (int, long)))

    def test_resynchronize(self):
        """
        On the reactor "resynchronize" event, new mount-info messages
        should be sent.
        """
        def statvfs(path):
            return (4096, 0, mb(1000), mb(100), 0, 0, 0, 0, 0)
        filename = self.makeFile("""\
/dev/hda1 / ext3 rw 0 0
""")
        plugin = self.get_mount_info(mounts_file=filename,
                                     create_time=self.reactor.time,
                                     statvfs=statvfs, mtab_file=filename)
        self.monitor.add(plugin)

        plugin.run()
        plugin.exchange()
        self.reactor.fire("resynchronize", scopes=["disk"])
        plugin.run()
        plugin.exchange()
        messages = self.mstore.get_pending_messages()
        messages = [message for message in messages
                    if message["type"] == "mount-info"]
        expected_message = {
            "type": "mount-info",
            "mount-info": [(0, {"device": "/dev/hda1", "mount-point": "/",
                                "total-space": 4096000,
                                "filesystem": "ext3"})]}
        self.assertMessages(messages, [expected_message, expected_message])

    def test_bind_mounts(self):
        """
        Mounted devices that are mounted using Linux's "--bind" option
        shouldn't be listed, as they have the same free space/used space as the
        device they're bound to.
        """
        def statvfs(path):
            return (4096, 0, mb(1000), mb(100), 0, 0, 0, 0, 0)

        # From this test data, we expect only two mount points to be returned,
        # and the other two to be ignored (the rebound /dev/hda2 -> /mnt
        # mounting)
        filename = self.makeFile("""\
/dev/devices/by-uuid/12345567 / ext3 rw 0 0
/dev/hda2 /usr ext3 rw 0 0
/dev/devices/by-uuid/12345567 /mnt ext3 rw 0 0
/dev/devices/by-uuid/12345567 /media/Boot\\040OSX hfsplus rw 0 0
""")

        mtab_filename = self.makeFile("""\
/dev/hda1 / ext3 rw 0 0
/dev/hda2 /usr ext3 rw 0 0
/opt /mnt none rw,bind 0 0
/opt /media/Boot\\040OSX none rw,bind 0 0
""")
        plugin = MountInfo(mounts_file=filename, create_time=self.reactor.time,
                           statvfs=statvfs, mtab_file=mtab_filename)

        self.monitor.add(plugin)
        plugin.run()
        message = plugin.create_mount_info_message()
        self.assertEqual(message.get("mount-info"),
                         [(0, {"device": "/dev/devices/by-uuid/12345567",
                               "mount-point": "/", "total-space": 4096000L,
                               "filesystem": "ext3"}),
                          (0, {"device": "/dev/hda2",
                               "mount-point": "/usr",
                               "total-space": 4096000L,
                               "filesystem": "ext3"}),
                          ])

    def test_no_mtab_file(self):
        """
        If there's no mtab file available, then we can make no guesses about
        bind mounted directories, so any filesystems in /proc/mounts will be
        reported.
        """
        def statvfs(path):
            return (4096, 0, mb(1000), mb(100), 0, 0, 0, 0, 0)

        # In this test, we expect all mount points to be returned, as we can't
        # identify any as bind mounts.
        filename = self.makeFile("""\
/dev/devices/by-uuid/12345567 / ext3 rw 0 0
/dev/hda2 /usr ext3 rw 0 0
/dev/devices/by-uuid/12345567 /mnt ext3 rw 0 0
""")
        # mktemp isn't normally secure, due to race conditions, but in this
        # case, we don't actually create the file at all.
        mtab_filename = tempfile.mktemp()
        plugin = MountInfo(mounts_file=filename, create_time=self.reactor.time,
                           statvfs=statvfs, mtab_file=mtab_filename)
        self.monitor.add(plugin)
        plugin.run()
        message = plugin.create_mount_info_message()
        self.assertEqual(message.get("mount-info"),
                         [(0, {"device": "/dev/devices/by-uuid/12345567",
                               "mount-point": "/", "total-space": 4096000L,
                               "filesystem": "ext3"}),
                          (0, {"device": "/dev/hda2",
                               "mount-point": "/usr",
                               "total-space": 4096000L,
                               "filesystem": "ext3"}),
                          (0, {"device": "/dev/devices/by-uuid/12345567",
                               "mount-point": "/mnt",
                               "total-space": 4096000L,
                               "filesystem": "ext3"})])

    def test_no_message_if_not_accepted(self):
        """
        Don't add any messages at all if the broker isn't currently
        accepting their type.
        """
        self.mstore.set_accepted_types([])

        def statvfs(path):
            return (4096, 0, mb(1000), mb(100), 0, 0, 0, 0, 0)

        # From this test data, we expect only two mount points to be returned,
        # and the third to be ignored (the rebound /dev/hda2 -> /mnt mounting)
        filename = self.makeFile("""\
/dev/devices/by-uuid/12345567 / ext3 rw 0 0
/dev/hda2 /usr ext3 rw 0 0
/dev/devices/by-uuid/12345567 /mnt ext3 rw 0 0
""")

        mtab_filename = self.makeFile("""\
/dev/hda1 / ext3 rw 0 0
/dev/hda2 /usr ext3 rw 0 0
/opt /mnt none rw,bind 0 0
""")

        plugin = MountInfo(mounts_file=filename, create_time=self.reactor.time,
                           statvfs=statvfs, mtab_file=mtab_filename)
        self.monitor.add(plugin)
        self.reactor.advance(self.monitor.step_size * 2)
        self.monitor.exchange()

        self.mstore.set_accepted_types(["mount-info"])
        self.assertMessages(list(self.mstore.get_pending_messages()), [])

    def test_call_on_accepted(self):
        plugin = self.get_mount_info(create_time=self.reactor.time)
        self.monitor.add(plugin)

        self.reactor.advance(plugin.run_interval)

        remote_broker_mock = self.mocker.replace(self.remote)
        remote_broker_mock.send_message(ANY, ANY, urgent=True)
        self.mocker.result(succeed(None))
        self.mocker.count(2)
        self.mocker.replay()

        self.reactor.fire(("message-type-acceptance-changed", "mount-info"),
                          True)

    def test_persist_timing(self):
        """Mount info are only persisted when exchange happens.

        Previously mount info were persisted as soon as they were gathered: if
        an event happened between the persist and the exchange, the server
        didn't get the mount info at all. This test ensures that mount info are
        only saved when exchange happens.
        """
        def statvfs(path):
            return (4096, 0, mb(1000), mb(100), 0, 0, 0, 0, 0)

        filename = self.makeFile("""\
/dev/hda1 / ext3 rw 0 0
""")
        plugin = MountInfo(mounts_file=filename, create_time=self.reactor.time,
                           statvfs=statvfs, mtab_file=filename)
        self.monitor.add(plugin)
        plugin.run()
        message1 = plugin.create_mount_info_message()
        self.assertEqual(
            message1.get("mount-info"),
            [(0, {"device": "/dev/hda1",
                  "filesystem": "ext3",
                  "mount-point": "/",
                  "total-space": 4096000L})])
        plugin.run()
        message2 = plugin.create_mount_info_message()
        self.assertEqual(
            message2.get("mount-info"),
            [(0, {"device": "/dev/hda1",
                  "filesystem": "ext3",
                  "mount-point": "/",
                  "total-space": 4096000L})])
        # Run again, calling create_mount_info_message purge the information
        plugin.run()
        plugin.exchange()
        plugin.run()
        message3 = plugin.create_mount_info_message()
        self.assertIdentical(message3, None)

    def test_exchange_limits_exchanged_free_space_messages(self):
        """
        In order not to overload the server, the client should stagger the
        exchange of free-space messages.
        """
        def statvfs(path):
            return (4096, 0, mb(1000), mb(100), 0, 0, 0, 0, 0)

        filename = self.makeFile("""\
/dev/hda1 / ext3 rw 0 0
""")
        plugin = self.get_mount_info(mounts_file=filename, statvfs=statvfs,
                                     create_time=self.reactor.time,
                                     mtab_file=filename)
        # Limit the test exchange to 5 items.
        plugin.max_free_space_items_to_exchange = 5
        step_size = self.monitor.step_size
        self.monitor.add(plugin)

        # Exchange should trigger a flush of the persist database
        registry_mocker = self.mocker.replace(plugin.registry)
        registry_mocker.flush()
        self.mocker.result(None)
        self.mocker.replay()

        # Generate 10 data points
        self.reactor.advance(step_size * 10)
        self.monitor.exchange()

        messages = self.mstore.get_pending_messages()
        self.assertEqual(len(messages), 2)

        message = [d for d in messages if d["type"] == "free-space"][0]
        free_space = message["free-space"]
        free_space_items = len(free_space)
        self.assertEqual(free_space_items, 5)
        for i in range(free_space_items):
            self.assertEqual(free_space[i][0], (i + 1) * step_size)
            self.assertEqual(free_space[i][1], "/")
            self.assertEqual(free_space[i][2], 409600)

        # The second exchange should pick up the remaining items.
        self.mstore.delete_all_messages()
        self.monitor.exchange()

        messages = self.mstore.get_pending_messages()
        self.assertEqual(len(messages), 1)

        message = [d for d in messages if d["type"] == "free-space"][0]
        free_space = message["free-space"]
        free_space_items = len(free_space)
        self.assertEqual(free_space_items, 5)
        for i in range(free_space_items):
            # Note (i+6) we've already retrieved the first 5 items.
            self.assertEqual(free_space[i][0], (i + 6) * step_size)
            self.assertEqual(free_space[i][1], "/")
            self.assertEqual(free_space[i][2], 409600)

        # Third exchange should not get any further free-space messages
        self.mstore.delete_all_messages()
        self.monitor.exchange()
        messages = self.mstore.get_pending_messages()
        self.assertEqual(len(messages), 0)



from landscape.monitor.memoryinfo import MemoryInfo
from landscape.tests.helpers import LandscapeTest, MonitorHelper
from landscape.tests.mocker import ANY


class MemoryInfoTest(LandscapeTest):

    helpers = [MonitorHelper]

    SAMPLE_DATA = """
MemTotal:      1546436 kB
MemFree:         23452 kB
Buffers:         41656 kB
Cached:         807628 kB
SwapCached:      17572 kB
Active:        1030792 kB
Inactive:       426892 kB
HighTotal:           0 kB
HighFree:            0 kB
LowTotal:      1546436 kB
LowFree:         23452 kB
SwapTotal:     1622524 kB
SwapFree:      1604936 kB
Dirty:            1956 kB
Writeback:           0 kB
Mapped:         661772 kB
Slab:            54980 kB
CommitLimit:   2395740 kB
Committed_AS:  1566888 kB
PageTables:       2728 kB
VmallocTotal:   516088 kB
VmallocUsed:      5660 kB
VmallocChunk:   510252 kB
"""

    def setUp(self):
        super(MemoryInfoTest, self).setUp()

    def test_read_proc_meminfo(self):
        """
        When the memory info plugin runs it reads data from
        /proc/meminfo which it parses and accumulates to read values.
        This test ensures that /proc/meminfo is always parseable and
        that messages are in the expected format and contain data with
        expected datatypes.
        """
        plugin = MemoryInfo(create_time=self.reactor.time)
        self.monitor.add(plugin)

        self.reactor.advance(self.monitor.step_size)

        message = plugin.create_message()
        self.assertTrue("type" in message)
        self.assertEqual(message["type"], "memory-info")
        self.assertTrue("memory-info" in message)
        memory_info = message["memory-info"]
        self.assertEqual(len(memory_info), 1)
        self.assertTrue(isinstance(memory_info[0], tuple))
        self.assertTrue(len(memory_info), 3)
        self.assertTrue(isinstance(memory_info[0][0], int))
        self.assertTrue(isinstance(memory_info[0][1], int))
        self.assertTrue(isinstance(memory_info[0][2], int))

    def test_read_sample_data(self):
        """
        This test uses sample /proc/meminfo data and ensures that
        messages contain expected free memory and free swap values.
        """
        filename = self.makeFile(self.SAMPLE_DATA)
        plugin = MemoryInfo(source_filename=filename,
                            create_time=self.reactor.time)
        step_size = self.monitor.step_size
        self.monitor.add(plugin)

        self.reactor.advance(step_size)

        message = plugin.create_message()
        self.assertEqual(message["memory-info"][0], (step_size, 852, 1567))

    def test_messaging_flushes(self):
        """
        Duplicate message should never be created.  If no data is
        available, a message with an empty C{memory-info} list is
        expected.
        """
        filename = self.makeFile(self.SAMPLE_DATA)
        plugin = MemoryInfo(source_filename=filename,
                            create_time=self.reactor.time)
        self.monitor.add(plugin)

        self.reactor.advance(self.monitor.step_size)

        message = plugin.create_message()
        self.assertEqual(len(message["memory-info"]), 1)

        message = plugin.create_message()
        self.assertEqual(len(message["memory-info"]), 0)

    def test_ranges_remain_contiguous_after_flush(self):
        """
        The memory info plugin uses the accumulate function to queue
        messages.  Timestamps should always be contiguous, and always
        fall on a step boundary.
        """
        filename = self.makeFile(self.SAMPLE_DATA)
        plugin = MemoryInfo(source_filename=filename,
                            create_time=self.reactor.time)
        self.monitor.add(plugin)

        step_size = self.monitor.step_size
        for i in range(1, 10):
            self.reactor.advance(step_size)
            message = plugin.create_message()
            memory_info = message["memory-info"]
            self.assertEqual(len(memory_info), 1)
            self.assertEqual(memory_info[0][0], step_size * i)

    def test_never_exchange_empty_messages(self):
        """
        The plugin will create a message with an empty C{memory-info}
        list when no data is available.  If an empty message is
        created during exchange, it should not be queued.
        """
        self.mstore.set_accepted_types(["memory-info"])

        filename = self.makeFile(self.SAMPLE_DATA)
        plugin = MemoryInfo(source_filename=filename,
                            create_time=self.reactor.time)
        self.monitor.add(plugin)
        self.monitor.exchange()
        self.assertEqual(len(self.mstore.get_pending_messages()), 0)

    def test_exchange_messages(self):
        """
        The memory info plugin queues messages when manager.exchange()
        is called.  Each message should be aligned to a step boundary;
        messages collected between exchange period should be delivered
        in a single message.
        """
        self.mstore.set_accepted_types(["memory-info"])

        filename = self.makeFile(self.SAMPLE_DATA)
        plugin = MemoryInfo(source_filename=filename,
                            create_time=self.reactor.time)
        step_size = self.monitor.step_size
        self.monitor.add(plugin)

        self.reactor.advance(step_size * 2)
        self.monitor.exchange()

        self.assertMessages(self.mstore.get_pending_messages(),
                            [{"type": "memory-info",
                              "memory-info": [(step_size, 852, 1567),
                                              (step_size * 2, 852, 1567)]}])

    def test_call_on_accepted(self):
        plugin = MemoryInfo(source_filename=self.makeFile(self.SAMPLE_DATA),
                            create_time=self.reactor.time)
        self.monitor.add(plugin)

        self.reactor.advance(self.monitor.step_size * 1)

        remote_broker_mock = self.mocker.replace(self.remote)
        remote_broker_mock.send_message(ANY, ANY, urgent=True)
        self.mocker.replay()

        self.reactor.fire(("message-type-acceptance-changed", "memory-info"),
                          True)

    def test_no_message_if_not_accepted(self):
        """
        Don't add any messages at all if the broker isn't currently
        accepting their type.
        """
        plugin = MemoryInfo(create_time=self.reactor.time)
        self.monitor.add(plugin)
        self.reactor.advance(self.monitor.step_size)
        self.monitor.exchange()

        self.mstore.set_accepted_types(["memory-info"])
        self.assertMessages(list(self.mstore.get_pending_messages()), [])



import os

from twisted.internet.utils import getProcessOutput

from landscape.manager.plugin import ManagerPlugin


class HardwareInfo(ManagerPlugin):
    """A plugin to retrieve hardware information."""

    message_type = "hardware-info"
    run_interval = 60 * 60 * 24
    run_immediately = True
    command = "/usr/bin/lshw"

    def register(self, registry):
        super(HardwareInfo, self).register(registry)
        self.call_on_accepted(self.message_type, self.send_message)

    def run(self):
        return self.registry.broker.call_if_accepted(
            self.message_type, self.send_message)

    def send_message(self):
        result = getProcessOutput(
            self.command, args=["-xml", "-quiet"], env=os.environ, path=None)
        return result.addCallback(self._got_output)

    def _got_output(self, output):
        message = {"type": self.message_type, "data": output}
        return self.registry.broker.send_message(message, self._session_id)



import shutil
import tempfile

from twisted.internet.utils import getProcessOutputAndValue


class InvalidGPGSignature(Exception):
    """Raised when the gpg signature for a given file is invalid."""


def gpg_verify(filename, signature, gpg="/usr/bin/gpg"):
    """Verify the GPG signature of a file.

    @param filename: Path to the file to verify the signature against.
    @param signature: Path to signature to use.
    @param gpg: Optionally, path to the GPG binary to use.
    @return: a C{Deferred} resulting in C{True} if the signature is
            valid, C{False} otherwise.
        """

    def remove_gpg_home(ignored):
        shutil.rmtree(gpg_home)
        return ignored

    def check_gpg_exit_code((out, err, code)):
        if code != 0:
            raise InvalidGPGSignature("%s failed (out='%s', err='%s', "
                                      "code='%d')" % (gpg, out, err, code))

    gpg_home = tempfile.mkdtemp()
    args = ("--no-options", "--homedir", gpg_home, "--no-default-keyring",
            "--ignore-time-conflict", "--keyring", "/etc/apt/trusted.gpg",
            "--verify", signature, filename)

    result = getProcessOutputAndValue(gpg, args=args)
    result.addBoth(remove_gpg_home)
    result.addCallback(check_gpg_exit_code)
    return result



import dbus

from landscape.tests.helpers import LandscapeTest
from landscape.ui.tests.helpers import dbus_test_should_skip, dbus_skip_message
if not dbus_test_should_skip:
    from landscape.ui.model.registration.mechanism import (
        RegistrationMechanism, INTERFACE_NAME)


class MechanismTest(LandscapeTest):
    """
    L{MechanismTest} mocks out the actual registration process and allows us to
    simply and quickly check the outputs of registration that are relied on
    elsewhere.
    """

    def setUp(self):
        super(MechanismTest, self).setUp()
        dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)
        bus = dbus.SessionBus(private=True)
        self.bus_name = dbus.service.BusName(INTERFACE_NAME, bus)
        self.mechanism = None

    def tearDown(self):
        if not self.mechanism is None:
            self.mechanism.remove_from_connection()
        super(MechanismTest, self).tearDown()

    def make_fake_registration(self, succeed, message=""):
        """
        Return a fake registration method that will fail or succeed by
        returning L{succeed} (a boolean).
        """

        def _do_registration(this, config_path):
            return succeed, message

        return _do_registration

    def make_fake_disabling(self, succeed):
        """
        Return a fake disabling method that will fail or succeed by
        returning L{succeed} (a boolean).
        """

        def _do_disabling(this):
            return succeed

        return _do_disabling

    def test_registration_succeed(self):
        """
        Test we get appropriate feedback from a successful connection when we
        call L{register} synchronously.
        """
        RegistrationMechanism._do_registration = self.make_fake_registration(
            True)
        self.mechanism = RegistrationMechanism(self.bus_name)
        self.assertEqual(
            (True, "Registration message sent to Landscape server.\n"),
            self.mechanism.register("foo"))

    def test_registration_fail(self):
        """
        Test we get appropriate feedback from a failed connection when we
        call L{register} synchronously.
        """
        RegistrationMechanism._do_registration = self.make_fake_registration(
            False, "boom")
        self.mechanism = RegistrationMechanism(self.bus_name)
        self.assertEqual((False, "boom"),
                         self.mechanism.register("foo"))

    def test_disabling_succeed(self):
        """
        Test we get True from a failed disabling when we call L{disable}
        synchronously.
        """
        RegistrationMechanism._do_disabling = self.make_fake_disabling(True)
        self.mechanism = RegistrationMechanism(self.bus_name)
        self.assertTrue(self.mechanism.disable())

    def test_disabling_fail(self):
        """
        Test we get False from a failed disabling when we call L{disable}
        synchronously.
        """
        RegistrationMechanism._do_disabling = self.make_fake_disabling(False)
        self.mechanism = RegistrationMechanism(self.bus_name)
        self.assertFalse(self.mechanism.disable())

    def test_exit(self):
        """
        Test that we cause the mechanism to exit.
        """
        self.mechanism = RegistrationMechanism(self.bus_name)
        self.assertRaises(SystemExit, self.mechanism.exit)

    if dbus_test_should_skip:
        skip = dbus_skip_message



import os
import json
import logging

from landscape.lib.fs import read_file


def get_juju_info(config):
    """
    Returns available Juju info or C{None} if the path referenced from
    L{config} is not a valid file.
    """
    if not os.path.exists(config.juju_filename):
        return

    json_contents = read_file(config.juju_filename)
    try:
        juju_info = json.loads(json_contents)
    # Catch any error the json lib could throw, because we don't know or
    # care what goes wrong - we'll display a generic error message and
    # return None in any case.
    except Exception:
        logging.exception(
            "Error attempting to read JSON from %s" % config.juju_filename)
        return None

    juju_info["api-addresses"] = juju_info["api-addresses"].split()
    return juju_info



from landscape.tests.helpers import LandscapeTest
from landscape.manager.config import ManagerConfiguration, ALL_PLUGINS
from landscape.manager.scriptexecution import ALL_USERS


class ManagerConfigurationTest(LandscapeTest):

    def setUp(self):
        super(ManagerConfigurationTest, self).setUp()
        self.config = ManagerConfiguration()

    def test_plugin_factories(self):
        """By default all plugins are enabled."""
        self.assertEqual(["ProcessKiller", "PackageManager", "UserManager",
                          "ShutdownManager", "AptSources", "HardwareInfo",
                          "KeystoneToken", "HAService"],
                         ALL_PLUGINS)
        self.assertEqual(ALL_PLUGINS, self.config.plugin_factories)

    def test_plugin_factories_with_manager_plugins(self):
        """
        The C{--manager-plugins} command line option can be used to specify
        which plugins should be active.
        """
        self.config.load(["--manager-plugins", "ProcessKiller"])
        self.assertEqual(self.config.plugin_factories, ["ProcessKiller"])

    def test_include_script_execution(self):
        """
        Extra plugins can be specified with the C{--include-manager-plugins}
        command line option.
        """
        self.config.load(["--include-manager-plugins", "ScriptExecution"])
        self.assertEqual(len(self.config.plugin_factories),
                         len(ALL_PLUGINS) + 1)
        self.assertTrue('ScriptExecution' in self.config.plugin_factories)

    def test_get_allowed_script_users(self):
        """
        If no script users are specified, the default is 'nobody'.
        """
        self.assertEqual(self.config.get_allowed_script_users(), ["nobody"])

    def test_get_allowed_script_users_all(self):
        """
        When script_users is C{ALL}, C{get_allowed_script_users} returns
        L{ALL_USERS}.
        """
        self.config.load(["--script-users", "\tALL "])
        self.assertIs(self.config.get_allowed_script_users(), ALL_USERS)

    def test_get_allowed_script_users_with_users(self):
        """
        It's possible to specify a list of usernames to allow scripts to run
        as.
        """
        self.config.load(["--script-users", "foo, bar,baz"])
        self.assertEqual(self.config.get_allowed_script_users(),
                         ["foo", "bar", "baz"])



from twisted.internet.defer import succeed

from landscape.monitor.swiftusage import SwiftUsage
from landscape.tests.helpers import LandscapeTest, MonitorHelper
from landscape.tests.mocker import ANY


class FakeRing(object):
    def __init__(self, ip_port_tuples=[]):
        self.devs = [
            {"ip": ip, "port": port}
            for ip, port in ip_port_tuples]


class SwiftUsageTest(LandscapeTest):
    """Tests for swift-usage plugin."""

    helpers = [MonitorHelper]

    def setUp(self):
        LandscapeTest.setUp(self)
        self.mstore.set_accepted_types(["swift-usage"])
        self.plugin = SwiftUsage(
            create_time=self.reactor.time, swift_ring=self.makeFile("ring"))
        self.plugin._has_swift = True

    def test_wb_should_run_not_active(self):
        """
        L{SwiftUsage._should_run} returns C{False} if plugin is not active.
        """
        plugin = SwiftUsage(create_time=self.reactor.time)
        plugin.active = False
        self.assertFalse(plugin._should_run())

    def test_wb_should_run_no_swift(self):
        """
        L{SwiftUsage._should_run} returns C{False} if Swift client library is
        not available. It also disables the plugin.
        """
        plugin = SwiftUsage(create_time=self.reactor.time)
        plugin._has_swift = False
        self.assertFalse(plugin._should_run())
        self.assertFalse(plugin.active)

    def test_wb_should_run_no_swift_ring(self):
        """
        L{SwiftUsage._should_run} returns C{False} if Swift ring configuration
        file is not found.
        """
        plugin = SwiftUsage(
            create_time=self.reactor.time, swift_ring=self.makeFile())
        plugin._has_swift = True
        self.assertFalse(plugin._should_run())

    def test_wb_should_run(self):
        """
        L{SwiftUsage._should_run} returns C{True} if everything if the Swift
        ring is properly configured.
        """
        plugin = SwiftUsage(
            create_time=self.reactor.time, swift_ring=self.makeFile("ring"))
        plugin._has_swift = True
        self.assertTrue(plugin._should_run())

    def test_exchange_messages(self):
        """
        The plugin queues message when manager.exchange() is called.
        Each message should be aligned to a step boundary; only a sing message
        with the latest swift device information will be delivered in a single
        message.
        """
        points = [(1234, "sdb", 100000, 80000, 20000),
                  (1234, "sdc", 200000, 120000, 800000)]
        self.plugin._swift_usage_points = points

        self.monitor.add(self.plugin)
        self.monitor.exchange()

        messages = self.mstore.get_pending_messages()
        self.assertEqual(len(messages), 1)
        data_points = messages[0]["data-points"]
        self.assertEqual(points, data_points)

    def test_no_exchange_empty_messages(self):
        """
        If no usage data is available, no message is exchanged.
        """
        self.monitor.add(self.plugin)
        self.monitor.exchange()

        self.assertEqual([], self.mstore.get_pending_messages())

    def test_create_message(self):
        """L{SwiftUsage.create_message} returns a 'swift-usage' message."""
        points = [(1234, "sdb", 100000, 80000, 20000),
                  (1234, "sdc", 200000, 120000, 80000)]
        self.plugin._swift_usage_points = points
        message = self.plugin.create_message()
        self.assertEqual(
            {"type": "swift-usage", "data-points": points}, message)

    def test_create_message_empty(self):
        """
        L{SwiftUsage.create_message} returns C{None} if no data are available.
        """
        self.assertIs(None, self.plugin.create_message())

    def test_crate_message_flushes(self):
        """Duplicate message should never be created."""
        self.monitor.add(self.plugin)
        self.plugin._swift_usage_points = [(1234, "sdb", 100000, 80000, 20000)]
        self.plugin._get_recon_host = lambda: ("192.168.1.10", 6000)

        self.reactor.advance(self.monitor.step_size)
        message = self.plugin.create_message()
        self.assertIsNot(None, message)
        message = self.plugin.create_message()
        self.assertIs(None, message)

    def test_no_message_if_not_accepted(self):
        """
        No message is sent if the broker isn't currently accepting their type.
        """
        self.plugin._swift_usage_points = [(1234, "sdb", 100000, 80000, 20000)]
        self.plugin._get_recon_host = lambda: ("192.168.1.10", 6000)

        self.mstore.set_accepted_types([])
        self.monitor.add(self.plugin)

        self.reactor.advance(self.monitor.step_size * 2)
        self.monitor.exchange()

        self.assertMessages(list(self.mstore.get_pending_messages()), [])

    def test_call_on_accepted(self):
        """
        When message type acceptance is added for 'swift' message,
        send_message gets called.
        """
        self.plugin._swift_usage_points = [(1234, "sdb", 100000, 80000, 20000)]
        self.plugin._get_recon_host = lambda: ("192.168.1.10", 6000)

        self.monitor.add(self.plugin)
        self.reactor.advance(self.plugin.run_interval)

        remote_broker_mock = self.mocker.replace(self.remote)
        remote_broker_mock.send_message(ANY, ANY, urgent=True)
        self.mocker.result(succeed(None))
        self.mocker.count(1)  # 1 send message is called
        self.mocker.replay()

        self.reactor.fire(
            ("message-type-acceptance-changed", "swift-usage"), True)

    def test_message_only_mounted_devices(self):
        """
        The plugin only collects usage for mounted devices.
        """
        recon_response = [
            {"device": "vdb",
             "mounted": True,
             "size": 100000,
             "avail": 80000,
             "used": 20000},
            {"device": "vdc",
             "mounted": False,
             "size": "",
             "avail": "",
             "used": ""},
            {"device": "vdd",
             "mounted": True,
             "size": 200000,
             "avail": 10000,
             "used": 190000}]
        self.plugin._perform_recon_call = lambda host: succeed(recon_response)
        self.plugin._get_recon_host = lambda: ("192.168.1.10", 6000)

        self.monitor.add(self.plugin)
        self.reactor.advance(self.plugin._interval)
        self.plugin._handle_usage(recon_response)

        self.assertEqual(
            [(30, "vdb", 100000, 80000, 20000),
             (30, "vdd", 200000, 10000, 190000)],
            self.plugin._swift_usage_points)
        self.assertEqual(["vdb", "vdd"], self.plugin._persist.get("devices"))
        self.assertNotIn("vdc", self.plugin._persist.get("usage"))

    def test_message_remove_disappeared_devices(self):
        """
        Usage for devices that have disappeared are removed from the persist.
        """
        recon_response = [
            {"device": "vdb",
             "mounted": True,
             "size": 100000,
             "avail": 80000,
             "used": 20000},
            {"device": "vdc",
             "mounted": True,
             "size": 200000,
             "avail": 10000,
             "used": 190000}]
        self.plugin._perform_recon_call = lambda host: succeed(recon_response)
        self.plugin._get_recon_host = lambda: ("192.168.1.10", 6000)

        self.monitor.add(self.plugin)
        self.reactor.advance(self.monitor.step_size)
        self.plugin._handle_usage(recon_response)
        self.assertEqual(
            ["vdb", "vdc"], sorted(self.plugin._persist.get("devices")))

        recon_response = [
            {"device": "vdb",
             "mounted": True,
             "size": 100000,
             "avail": 70000,
             "used": 30000}]
        self.reactor.advance(self.monitor.step_size)
        self.plugin._handle_usage(recon_response)
        self.assertNotIn("vdc", self.plugin._persist.get("usage"))
        self.assertEqual(["vdb"], self.plugin._persist.get("devices"))

    def test_message_remove_unmounted_devices(self):
        """
        Usage for devices that are no longer mounted are removed from the
        persist.
        """
        recon_response = [
            {"device": "vdb",
             "mounted": True,
             "size": 100000,
             "avail": 80000,
             "used": 20000},
            {"device": "vdc",
             "mounted": True,
             "size": 200000,
             "avail": 10000,
             "used": 190000}]
        self.plugin._perform_recon_call = lambda host: succeed(recon_response)
        self.plugin._get_recon_host = lambda: ("192.168.1.10", 6000)

        self.monitor.add(self.plugin)
        self.reactor.advance(self.monitor.step_size)
        self.plugin._handle_usage(recon_response)
        self.assertEqual(
            ["vdb", "vdc"], sorted(self.plugin._persist.get("devices")))

        recon_response = [
            {"device": "vdb",
             "mounted": True,
             "size": 100000,
             "avail": 70000,
             "used": 30000},
            {"device": "vdc",
             "mounted": False,
             "size": "",
             "avail": "",
             "used": ""}]
        self.reactor.advance(self.monitor.step_size)
        self.plugin._handle_usage(recon_response)
        self.assertNotIn("vdc", self.plugin._persist.get("usage"))
        self.assertEqual(["vdb"], self.plugin._persist.get("devices"))



import fcntl
import time
import os


class LockError(Exception):
    """Raised when unable to lock a file."""


def lock_path(path, timeout=0):
    fd = os.open(path, os.O_CREAT)
    flags = fcntl.fcntl(fd, fcntl.F_GETFD, 0)
    flags |= fcntl.FD_CLOEXEC
    fcntl.fcntl(fd, fcntl.F_SETFD, flags)

    started = time.time()

    while True:
        try:
            fcntl.flock(fd, fcntl.LOCK_EX | fcntl.LOCK_NB)
        except IOError:
            if started < time.time() - timeout:
                raise LockError("Couldn't obtain lock")
        else:
            break
        time.sleep(0.1)

    def unlock_path():
        fcntl.flock(fd, fcntl.LOCK_UN)
        os.close(fd)

    return unlock_path



import dbus

from landscape.tests.helpers import LandscapeTest
from landscape.ui.tests.helpers import dbus_test_should_skip, dbus_skip_message
if not dbus_test_should_skip:
    from landscape.ui.model.registration.mechanism import (
        RegistrationMechanism, INTERFACE_NAME)
    from landscape.ui.model.registration.proxy import RegistrationProxy


class TimeoutTest(LandscapeTest):
    """
    L{TimeoutTest} bypasses DBus and tests with a faked method that raises a
    timeout exception.
    """

    def setUp(self):
        super(TimeoutTest, self).setUp()
        self.error_handler_messages = []

        class FakeBus(object):
            """
            Hello, I will be your fake DBus for this flight.
            """

        class FakeTimeoutException(dbus.DBusException):
            _dbus_error_name = "org.freedesktop.DBus.Error.NoReply"

        class FakeFailyMechanism(object):

            def register(this, config_path, reply_handler=None,
                         error_handler=None):
                raise FakeTimeoutException()

        def fake_setup_interface(this, bus):
            this._interface = FakeFailyMechanism()
            this._bus = bus

        def fake_register_handlers(this):
            pass

        def fake_remove_handlers(this):
            pass

        def fake_error_handler(message):
            self.error_handler_messages.append(message)

        RegistrationProxy._setup_interface = fake_setup_interface
        RegistrationProxy._register_handlers = fake_register_handlers
        RegistrationProxy._remove_handlers = fake_remove_handlers
        self.proxy = RegistrationProxy(bus=FakeBus(),
                                       on_register_error=fake_error_handler)

    def tearDown(self):
        self.error_handler_messages = []
        super(TimeoutTest, self).tearDown()

    def test_register(self):
        """
        Test that the proxy calls through to the underlying interface and
        correctly performs registration.
        """
        self.proxy.register("foo")
        self.assertEqual(1, len(self.error_handler_messages))
        [message] = self.error_handler_messages
        self.assertEqual("Registration timed out.", message)

    if dbus_test_should_skip:
        skip = dbus_skip_message


class RegistrationProxyTest(LandscapeTest):
    """
    L{RegistrationProxyTest} bypasses DBus to simply check the interface
    between the proxy and the mechanism it would usually contact via DBus.
    """

    def setUp(self):
        super(RegistrationProxyTest, self).setUp()
        dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)
        bus = dbus.SessionBus(private=True)
        bus_name = dbus.service.BusName(INTERFACE_NAME, bus)

        def fake_do__registration(this, config_path):
            return True, ""

        def fake_do__disabling(this):
            return True

        RegistrationMechanism._do_registration = fake_do__registration
        RegistrationMechanism._do_disabling = fake_do__disabling
        self.mechanism = RegistrationMechanism(bus_name)

        def fake_setup_interface(this, bus):
            """
            This just allows us to test without actually relying on dbus.
            """
            this._interface = self.mechanism

        def fake_register_handlers(this):
            pass

        def fake_remove_handlers(this):
            pass

        def fake_callback(message=None):
            pass

        RegistrationProxy._setup_interface = fake_setup_interface
        RegistrationProxy._register_handlers = fake_register_handlers
        RegistrationProxy._remove_handlers = fake_remove_handlers
        self.proxy = RegistrationProxy(fake_callback, fake_callback,
                                       fake_callback, fake_callback,
                                       fake_callback, fake_callback)

    def tearDown(self):
        self.mechanism.remove_from_connection()
        super(RegistrationProxyTest, self).tearDown()

    def test_register(self):
        """
        Test that the proxy calls through to the underlying interface and
        correctly performs registration.
        """
        self.assertEqual(True, self.proxy.register("foo"))

    def test_disable(self):
        """
        Test that the proxy calls through to the underlying interface and
        correctly performs disabling.
        """
        self.assertEqual(True, self.proxy.disable())

    def test_exit(self):
        """
        Test that we can cause the mechanism to exit.
        """
        self.assertRaises(SystemExit, self.proxy.exit)

    if dbus_test_should_skip:
        skip = dbus_skip_message



"""A schema system. Yes. Another one!"""


class InvalidError(Exception):
    """Raised when invalid input is received."""
    pass


class Constant(object):
    """Something that must be equal to a constant value."""
    def __init__(self, value):
        self.value = value

    def coerce(self, value):
        if value != self.value:
            raise InvalidError("%r != %r" % (value, self.value))
        return value


class Any(object):
    """Something which must apply to any of a number of different schemas.

    @param schemas: Other schema objects.
    """
    def __init__(self, *schemas):
        self.schemas = schemas

    def coerce(self, value):
        """
        The result of the first schema which doesn't raise
        L{InvalidError} from its C{coerce} method will be returned.
        """
        for schema in self.schemas:
            try:
                return schema.coerce(value)
            except InvalidError:
                pass
        raise InvalidError("%r did not match any schema in %s"
                           % (value, self.schemas))


class Bool(object):
    """Something that must be a C{bool}."""
    def coerce(self, value):
        if not isinstance(value, bool):
            raise InvalidError("%r is not a bool" % (value,))
        return value


class Int(object):
    """Something that must be an C{int} or C{long}."""
    def coerce(self, value):
        if not isinstance(value, (int, long)):
            raise InvalidError("%r isn't an int or long" % (value,))
        return value


class Float(object):
    """Something that must be an C{int}, C{long}, or C{float}."""
    def coerce(self, value):
        if not isinstance(value, (int, long, float)):
            raise InvalidError("%r isn't a float" % (value,))
        return value


class Bytes(object):
    """A binary string."""
    def coerce(self, value):
        if not isinstance(value, str):
            raise InvalidError("%r isn't a str" % (value,))
        return value


class Unicode(object):
    """Something that must be a C{unicode}.

    If the value is a C{str}, it will automatically be decoded.

    @param encoding: The encoding to automatically decode C{str}s with.
    """

    def __init__(self, encoding="utf-8"):
        self.encoding = encoding

    def coerce(self, value):
        if isinstance(value, str):
            try:
                value = value.decode(self.encoding)
            except UnicodeDecodeError, e:
                raise InvalidError("%r can't be decoded: %s" % (value, str(e)))
        if not isinstance(value, unicode):
            raise InvalidError("%r isn't a unicode" % (value,))
        return value


class List(object):
    """Something which must be a C{list}.

    @param schema: The schema that all values of the list must match.
    """
    def __init__(self, schema):
        self.schema = schema

    def coerce(self, value):
        if not isinstance(value, list):
            raise InvalidError("%r is not a list" % (value,))
        new_list = list(value)
        for i, subvalue in enumerate(value):
            try:
                new_list[i] = self.schema.coerce(subvalue)
            except InvalidError, e:
                raise InvalidError(
                    "%r could not coerce with %s: %s"
                    % (subvalue, self.schema, e))
        return new_list


class Tuple(object):
    """Something which must be a fixed-length tuple.

    @param schema: A sequence of schemas, which will be applied to
        each value in the tuple respectively.
    """

    def __init__(self, *schema):
        self.schema = schema

    def coerce(self, value):
        if not isinstance(value, tuple):
            raise InvalidError("%r is not a tuple" % (value,))
        if len(value) != len(self.schema):
            raise InvalidError("Need %s items, got %s in %r"
                               % (len(self.schema), len(value), value))
        new_value = []
        for schema, value in zip(self.schema, value):
            new_value.append(schema.coerce(value))
        return tuple(new_value)


class KeyDict(object):
    """Something which must be a C{dict} with defined keys.

    The keys must be constant and the values must match a per-key schema.

    @param schema: A dict mapping keys to schemas that the values of those
        keys must match.
    """
    def __init__(self, schema, optional=None):
        if optional is None:
            optional = []
        self.optional = set(optional)
        self.schema = schema

    def coerce(self, value):
        new_dict = {}
        if not isinstance(value, dict):
            raise InvalidError("%r is not a dict." % (value,))
        for k, v in value.iteritems():
            if k not in self.schema:
                raise InvalidError("%r is not a valid key as per %r"
                                   % (k, self.schema))
            try:
                new_dict[k] = self.schema[k].coerce(v)
            except InvalidError, e:
                raise InvalidError(
                    "Value of %r key of dict %r could not coerce with %s: %s"
                    % (k, value, self.schema[k], e))
        new_keys = set(new_dict.keys())
        required_keys = set(self.schema.keys()) - self.optional
        missing = required_keys - new_keys
        if missing:
            raise InvalidError("Missing keys %s" % (missing,))
        return new_dict


class Dict(object):
    """Something which must be a C{dict} with arbitrary keys.

    @param key_schema: The schema that keys must match.
    @param value_schema: The schema that values must match.
    """

    def __init__(self, key_schema, value_schema):
        self.key_schema = key_schema
        self.value_schema = value_schema

    def coerce(self, value):
        if not isinstance(value, dict):
            raise InvalidError("%r is not a dict." % (value,))
        new_dict = {}
        for k, v in value.items():
            new_dict[self.key_schema.coerce(k)] = self.value_schema.coerce(v)
        return new_dict


class Message(KeyDict):
    """
    Like L{KeyDict}, but with three predefined keys: C{type}, C{api},
    and C{timestamp}. Of these, C{api} and C{timestamp} are optional.


    @param type: The type of the message. The C{type} key will need to
        match this as a constant.
    @param schema: A dict of additional schema in a format L{KeyDict}
        will accept.
    @param optional: An optional list of keys that should be optional.
    @param api: The server API version needed to send this message,
        if C{None} any version is fine.
    """
    def __init__(self, type, schema, optional=None, api=None):
        self.type = type
        self.api = api
        schema["timestamp"] = Float()
        schema["api"] = Any(Bytes(), Constant(None))
        schema["type"] = Constant(type)
        if optional is not None:
            optional.extend(["timestamp", "api"])
        else:
            optional = ["timestamp", "api"]
        super(Message, self).__init__(schema, optional=optional)

    def coerce(self, value):
        for k in value.keys():
            if k not in self.schema:
                # We don't know about this field, just discard it. This
                # is useful when a client that introduced some new field
                # in a message talks to an older server, that don't understand
                # the new field yet.
                value.pop(k)
        return super(Message, self).coerce(value)



"""
A monitor that collects data on network activity, and sends messages
with the inbound/outbound traffic per interface per step interval.
"""

import time

from landscape.lib.network import get_network_traffic, is_64
from landscape.accumulate import Accumulator

from landscape.monitor.plugin import MonitorPlugin


class NetworkActivity(MonitorPlugin):
    """
    Collect data regarding a machine's network activity.
    """

    message_type = "network-activity"
    persist_name = message_type
    run_interval = 30
    _rollover_maxint = 0
    scope = "network"

    max_network_items_to_exchange = 200

    def __init__(self, network_activity_file="/proc/net/dev",
                 create_time=time.time):
        self._source_file = network_activity_file
        # accumulated values for sending out via message
        self._network_activity = {}
        # our last traffic sample for calculating a traffic delta
        self._last_activity = {}
        self._create_time = create_time
        # We don't rollover on 64 bits, as 16 exabytes is a lot.
        if not is_64():
            self._rollover_maxint = pow(2, 32)

    def register(self, registry):
        super(NetworkActivity, self).register(registry)
        self._accumulate = Accumulator(self._persist, self.registry.step_size)
        self.call_on_accepted("network-activity", self.exchange, True)

    def create_message(self):
        network_activity = {}
        items = 0
        for interface, data in list(self._network_activity.items()):
            if data:
                network_activity[interface] = []
                while data and items < self.max_network_items_to_exchange:
                    item = data.pop(0)
                    network_activity[interface].append(item)
                    items += 1
                if items >= self.max_network_items_to_exchange:
                    break
        if not network_activity:
            return
        return {"type": "network-activity", "activities": network_activity}

    def send_message(self, urgent):
        message = self.create_message()
        if not message:
            return
        self.registry.broker.send_message(
            message, self._session_id, urgent=urgent)

    def exchange(self, urgent=False):
        self.registry.broker.call_if_accepted("network-activity",
                                              self.send_message, urgent)

    def _traffic_delta(self, new_traffic):
        """
        Given network activity metrics across all interfaces, calculate
        and return the delta data transferred for inbound and outbound
        traffic. Returns a tuple of interface name, outbound delta,
        inbound delta.
        """
        for interface in new_traffic:
            traffic = new_traffic[interface]
            if interface in self._last_activity:
                previous_out, previous_in = self._last_activity[interface]
                delta_out = traffic["send_bytes"] - previous_out
                delta_in = traffic["recv_bytes"] - previous_in
                if delta_out < 0:
                    delta_out += self._rollover_maxint
                if delta_in < 0:
                    delta_in += self._rollover_maxint
                # If it's still zero or less, we discard the value. The next
                # value will be compared to the current traffic, and
                # hopefully things will catch up.
                if delta_out <= 0 and delta_in <= 0:
                    continue

                yield interface, delta_out, delta_in
            self._last_activity[interface] = (
                traffic["send_bytes"], traffic["recv_bytes"])
        for interface in self._last_activity.keys():
            if interface not in new_traffic:
                del self._last_activity[interface]

    def run(self):
        """
        Sample network traffic statistics and store them into the
        accumulator, recording step data.
        """
        new_timestamp = int(self._create_time())
        new_traffic = get_network_traffic(self._source_file)
        for interface, delta_out, delta_in in self._traffic_delta(new_traffic):
            out_step_data = self._accumulate(
                new_timestamp, delta_out, "delta-out-%s" % interface)

            in_step_data = self._accumulate(
                new_timestamp, delta_in, "delta-in-%s" % interface)

            # there's only data when we cross a step boundary
            if not (in_step_data and out_step_data):
                continue

            steps = self._network_activity.setdefault(interface, [])
            steps.append(
                (in_step_data[0], int(in_step_data[1]), int(out_step_data[1])))



"""Deployment code for the sysinfo tool."""
import os
import sys
from logging import getLogger, Formatter
from logging.handlers import RotatingFileHandler

from twisted.python.reflect import namedClass
from twisted.internet.defer import Deferred, maybeDeferred

from landscape.deployment import BaseConfiguration
from landscape.sysinfo.sysinfo import SysInfoPluginRegistry, format_sysinfo


ALL_PLUGINS = ["Load", "Disk", "Memory", "Temperature", "Processes",
               "LoggedInUsers", "LandscapeLink", "Network"]


class SysInfoConfiguration(BaseConfiguration):
    """Specialized configuration for the Landscape sysinfo tool."""

    default_config_filenames = ("/etc/landscape/client.conf",)
    if os.getuid() != 0:
        default_config_filenames += (
            os.path.expanduser("~/.landscape/sysinfo.conf"),)

    config_section = "sysinfo"

    def make_parser(self):
        """
        Specialize L{Configuration.make_parser}, adding any
        sysinfo-specific options.
        """
        parser = super(SysInfoConfiguration, self).make_parser()

        parser.add_option("--sysinfo-plugins", metavar="PLUGIN_LIST",
                          help="Comma-delimited list of sysinfo plugins to "
                               "use. Default is to use all plugins.")

        parser.add_option("--exclude-sysinfo-plugins", metavar="PLUGIN_LIST",
                          help="Comma-delimited list of sysinfo plugins to "
                               "NOT use. This always take precedence over "
                               "plugins to include.")

        parser.epilog = "Default plugins: %s" % (", ".join(ALL_PLUGINS))
        return parser

    def get_plugin_names(self, plugin_spec):
        return [x.strip() for x in plugin_spec.split(",")]

    def get_plugins(self):
        if self.sysinfo_plugins is None:
            include = ALL_PLUGINS
        else:
            include = self.get_plugin_names(self.sysinfo_plugins)
        if self.exclude_sysinfo_plugins is None:
            exclude = []
        else:
            exclude = self.get_plugin_names(self.exclude_sysinfo_plugins)
        plugins = [x for x in include if x not in exclude]
        return [namedClass("landscape.sysinfo.%s.%s"
                           % (plugin_name.lower(), plugin_name))()
                for plugin_name in plugins]


def get_landscape_log_directory(landscape_dir=None):
    """
    Work out the correct path to store logs in depending on the effective
    user id of the current process.
    """
    if landscape_dir is None:
        if os.getuid() == 0:
            landscape_dir = "/var/log/landscape"
        else:
            landscape_dir = os.path.expanduser("~/.landscape")
    return landscape_dir


def setup_logging(landscape_dir=None):
    landscape_dir = get_landscape_log_directory(landscape_dir)
    logger = getLogger("landscape-sysinfo")
    logger.propagate = False
    if not os.path.isdir(landscape_dir):
        os.mkdir(landscape_dir)
    log_filename = os.path.join(landscape_dir, "sysinfo.log")
    handler = RotatingFileHandler(log_filename,
                                  maxBytes=500 * 1024, backupCount=1)
    logger.addHandler(handler)
    handler.setFormatter(Formatter("%(asctime)s %(levelname)-8s %(message)s"))


def run(args, reactor=None, sysinfo=None):
    """
    @param reactor: The reactor to (optionally) run the sysinfo plugins in.
    """
    try:
        setup_logging()
    except IOError, e:
        sys.exit("Unable to setup logging. %s" % e)

    if sysinfo is None:
        sysinfo = SysInfoPluginRegistry()
    config = SysInfoConfiguration()
    # landscape-sysinfo needs to work where there's no
    # /etc/landscape/client.conf See lp:1293990
    config.load(args, accept_nonexistent_default_config=True)
    for plugin in config.get_plugins():
        sysinfo.add(plugin)

    def show_output(result):
        print format_sysinfo(sysinfo.get_headers(), sysinfo.get_notes(),
                             sysinfo.get_footnotes(), indent="  ")

    def run_sysinfo():
        return sysinfo.run().addCallback(show_output)

    if reactor is not None:
        # In case any plugins run processes or do other things that require the
        # reactor to already be started, we delay them until the reactor is
        # running.
        done = Deferred()
        reactor.callWhenRunning(
            lambda: maybeDeferred(run_sysinfo).chainDeferred(done))

        def stop_reactor(result):
            # We won't need to use callLater here once we use Twisted >8.
            # tm:3011
            reactor.callLater(0, reactor.stop)
            return result

        done.addBoth(stop_reactor)
        reactor.run()
    else:
        done = run_sysinfo()
    return done



import time

from landscape.lib.monitor import CoverageMonitor
from landscape.lib.sysstats import get_thermal_zones

from landscape.accumulate import Accumulator
from landscape.monitor.plugin import MonitorPlugin


class Temperature(MonitorPlugin):
    """Capture thermal zone temperatures and trip point settings."""

    persist_name = "temperature"
    scope = "temperature"
    # Prevent the Plugin base-class from scheduling looping calls.
    run_interval = None

    def __init__(self, interval=30, monitor_interval=60 * 60,
                 thermal_zone_path=None, create_time=time.time):
        self.thermal_zone_path = thermal_zone_path
        self._interval = interval
        self._monitor_interval = monitor_interval
        self._create_time = create_time
        self._thermal_zones = []
        self._temperatures = {}

        for thermal_zone in get_thermal_zones(self.thermal_zone_path):
            self._thermal_zones.append(thermal_zone.name)
            self._temperatures[thermal_zone.name] = []

    def register(self, registry):
        super(Temperature, self).register(registry)
        if self._thermal_zones:
            self._accumulate = Accumulator(self._persist,
                                           self.registry.step_size)

            registry.reactor.call_every(self._interval, self.run)

            self._monitor = CoverageMonitor(self._interval, 0.8,
                                            "temperature snapshot",
                                            create_time=self._create_time)
            registry.reactor.call_every(self._monitor_interval,
                                        self._monitor.log)
            registry.reactor.call_on("stop", self._monitor.log, priority=2000)
            self.call_on_accepted("temperature", self.exchange, True)

    def create_messages(self):
        messages = []
        for zone in self._thermal_zones:
            temperatures = self._temperatures[zone]
            self._temperatures[zone] = []
            if not temperatures:
                continue
            messages.append({"type": "temperature", "thermal-zone": zone,
                             "temperatures": temperatures})
        return messages

    def send_messages(self, urgent):
        for message in self.create_messages():
            self.registry.broker.send_message(
                message, self._session_id, urgent=urgent)

    def exchange(self, urgent=False):
        self.registry.broker.call_if_accepted("temperature",
                                              self.send_messages, urgent)

    def run(self):
        self._monitor.ping()
        now = int(self._create_time())
        for zone in get_thermal_zones(self.thermal_zone_path):
            if zone.temperature_value is not None:
                key = ("accumulate", zone.name)
                step_data = self._accumulate(now, zone.temperature_value, key)
                if step_data:
                    self._temperatures[zone.name].append(step_data)



#
#  subunit: extensions to python unittest to get test results from subprocesses.
#  Copyright (C) 2005  Robert Collins <robertc@robertcollins.net>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

import os
from StringIO import StringIO
import subprocess
import sys
import unittest

def test_suite():
    import subunit.tests
    return subunit.tests.test_suite()


def join_dir(base_path, path):
    """
    Returns an absolute path to C{path}, calculated relative to the parent
    of C{base_path}.

    @param base_path: A path to a file or directory.
    @param path: An absolute path, or a path relative to the containing
    directory of C{base_path}.

    @return: An absolute path to C{path}.
    """
    return os.path.join(os.path.dirname(os.path.abspath(base_path)), path)


class TestProtocolServer(object):
    """A class for receiving results from a TestProtocol client."""

    OUTSIDE_TEST = 0
    TEST_STARTED = 1
    READING_FAILURE = 2
    READING_ERROR = 3

    def __init__(self, client, stream=sys.stdout):
        """Create a TestProtocol server instance.

        client should be an object that provides
         - startTest
         - addSuccess
         - addFailure
         - addError
         - stopTest
        methods, i.e. a TestResult.
        """
        self.state = TestProtocolServer.OUTSIDE_TEST
        self.client = client
        self._stream = stream

    def _addError(self, offset, line):
        if (self.state == TestProtocolServer.TEST_STARTED and
            self.current_test_description == line[offset:-1]):
            self.state = TestProtocolServer.OUTSIDE_TEST
            self.current_test_description = None
            self.client.addError(self._current_test, RemoteError(""))
            self.client.stopTest(self._current_test)
            self._current_test = None
        elif (self.state == TestProtocolServer.TEST_STARTED and
            self.current_test_description + " [" == line[offset:-1]):
            self.state = TestProtocolServer.READING_ERROR
            self._message = ""
        else:
            self.stdOutLineReceived(line)

    def _addFailure(self, offset, line):
        if (self.state == TestProtocolServer.TEST_STARTED and
            self.current_test_description == line[offset:-1]):
            self.state = TestProtocolServer.OUTSIDE_TEST
            self.current_test_description = None
            self.client.addFailure(self._current_test, RemoteError())
            self.client.stopTest(self._current_test)
        elif (self.state == TestProtocolServer.TEST_STARTED and
            self.current_test_description + " [" == line[offset:-1]):
            self.state = TestProtocolServer.READING_FAILURE
            self._message = ""
        else:
            self.stdOutLineReceived(line)

    def _addSuccess(self, offset, line):
        if (self.state == TestProtocolServer.TEST_STARTED and
            self.current_test_description == line[offset:-1]):
            self.client.addSuccess(self._current_test)
            self.client.stopTest(self._current_test)
            self.current_test_description = None
            self._current_test = None
            self.state = TestProtocolServer.OUTSIDE_TEST
        else:
            self.stdOutLineReceived(line)

    def _appendMessage(self, line):
        if line[0:2] == " ]":
            # quoted ] start
            self._message += line[1:]
        else:
            self._message += line

    def endQuote(self, line):
        if self.state == TestProtocolServer.READING_FAILURE:
            self.state = TestProtocolServer.OUTSIDE_TEST
            self.current_test_description = None
            self.client.addFailure(self._current_test,
                                   RemoteError(self._message))
            self.client.stopTest(self._current_test)
        elif self.state == TestProtocolServer.READING_ERROR:
            self.state = TestProtocolServer.OUTSIDE_TEST
            self.current_test_description = None
            self.client.addError(self._current_test,
                                 RemoteError(self._message))
            self.client.stopTest(self._current_test)
        else:
            self.stdOutLineReceived(line)

    def lineReceived(self, line):
        """Call the appropriate local method for the received line."""
        if line == "]\n":
            self.endQuote(line)
        elif (self.state == TestProtocolServer.READING_FAILURE or
              self.state == TestProtocolServer.READING_ERROR):
            self._appendMessage(line)
        else:
            parts = line.split(None, 1)
            if len(parts) == 2:
                cmd, rest = parts
                offset = len(cmd) + 1
                cmd = cmd.strip(':')
                if cmd in ('test', 'testing'):
                    self._startTest(offset, line)
                elif cmd == 'error':
                    self._addError(offset, line)
                elif cmd == 'failure':
                    self._addFailure(offset, line)
                elif cmd in ('success', 'successful'):
                    self._addSuccess(offset, line)
                else:
                    self.stdOutLineReceived(line)
            else:
                self.stdOutLineReceived(line)

    def lostConnection(self):
        """The input connection has finished."""
        if self.state == TestProtocolServer.TEST_STARTED:
            self.client.addError(self._current_test,
                                 RemoteError("lost connection during test '%s'"
                                             % self.current_test_description))
            self.client.stopTest(self._current_test)
        elif self.state == TestProtocolServer.READING_ERROR:
            self.client.addError(self._current_test,
                                 RemoteError("lost connection during "
                                             "error report of test "
                                             "'%s'" %
                                             self.current_test_description))
            self.client.stopTest(self._current_test)
        elif self.state == TestProtocolServer.READING_FAILURE:
            self.client.addError(self._current_test,
                                 RemoteError("lost connection during "
                                             "failure report of test "
                                             "'%s'" %
                                             self.current_test_description))
            self.client.stopTest(self._current_test)

    def readFrom(self, pipe):
        for line in pipe.readlines():
            self.lineReceived(line)
        self.lostConnection()

    def _startTest(self, offset, line):
        """Internal call to change state machine. Override startTest()."""
        if self.state == TestProtocolServer.OUTSIDE_TEST:
            self.state = TestProtocolServer.TEST_STARTED
            self._current_test = RemotedTestCase(line[offset:-1])
            self.current_test_description = line[offset:-1]
            self.client.startTest(self._current_test)
        else:
            self.stdOutLineReceived(line)

    def stdOutLineReceived(self, line):
        self._stream.write(line)


class RemoteException(Exception):
    """An exception that occured remotely to python."""

    def __eq__(self, other):
        try:
            return self.args == other.args
        except AttributeError:
            return False


class TestProtocolClient(unittest.TestResult):
    """A class that looks like a TestResult and informs a TestProtocolServer."""

    def __init__(self, stream):
        unittest.TestResult.__init__(self)
        self._stream = stream

    def addError(self, test, error):
        """Report an error in test test."""
        self._stream.write("error: %s [\n" % test.shortDescription())
        for line in self._exc_info_to_string(error, test).splitlines():
            self._stream.write("%s\n" % line)
        self._stream.write("]\n")

    def addFailure(self, test, error):
        """Report a failure in test test."""
        self._stream.write("failure: %s [\n" % test.shortDescription())
        for line in self._exc_info_to_string(error, test).splitlines():
            self._stream.write("%s\n" % line)
        self._stream.write("]\n")

    def addSuccess(self, test):
        """Report a success in a test."""
        self._stream.write("successful: %s\n" % test.shortDescription())

    def startTest(self, test):
        """Mark a test as starting its test run."""
        self._stream.write("test: %s\n" % test.shortDescription())


def RemoteError(description=""):
    if description == "":
        description = "\n"
    return (RemoteException, RemoteException(description), None)


class RemotedTestCase(unittest.TestCase):
    """A class to represent test cases run in child processes."""

    def __eq__ (self, other):
        try:
            return self.__description == other.__description
        except AttributeError:
            return False

    def __init__(self, description):
        """Create a psuedo test case with description description."""
        self.__description = description

    def error(self, label):
        raise NotImplementedError("%s on RemotedTestCases is not permitted." %
            label)

    def setUp(self):
        self.error("setUp")

    def tearDown(self):
        self.error("tearDown")

    def shortDescription(self):
        return self.__description

    def id(self):
        return "%s.%s" % (self._strclass(), self.__description)

    def __str__(self):
        return "%s (%s)" % (self.__description, self._strclass())

    def __repr__(self):
        return "<%s description='%s'>" % \
               (self._strclass(), self.__description)

    def run(self, result=None):
        if result is None: result = self.defaultTestResult()
        result.startTest(self)
        result.addError(self, RemoteError("Cannot run RemotedTestCases.\n"))
        result.stopTest(self)

    def _strclass(self):
        cls = self.__class__
        return "%s.%s" % (cls.__module__, cls.__name__)


class ExecTestCase(unittest.TestCase):
    """A test case which runs external scripts for test fixtures."""

    def __init__(self, methodName='runTest'):
        """Create an instance of the class that will use the named test
           method when executed. Raises a ValueError if the instance does
           not have a method with the specified name.
        """
        unittest.TestCase.__init__(self, methodName)
        testMethod = getattr(self, methodName)
        self.script = join_dir(sys.modules[self.__class__.__module__].__file__,
                               testMethod.__doc__)

    def countTestCases(self):
        return 1

    def run(self, result=None):
        if result is None: result = self.defaultTestResult()
        self._run(result)

    def debug(self):
        """Run the test without collecting errors in a TestResult"""
        self._run(unittest.TestResult())

    def _run(self, result):
        protocol = TestProtocolServer(result)
        output = subprocess.Popen([self.script],
                                  stdout=subprocess.PIPE).communicate()[0]
        protocol.readFrom(StringIO(output))


class IsolatedTestCase(unittest.TestCase):
    """A TestCase which runs its tests in a forked process."""

    def run(self, result=None):
        if result is None: result = self.defaultTestResult()
        run_isolated(unittest.TestCase, self, result)


class IsolatedTestSuite(unittest.TestSuite):
    """A TestCase which runs its tests in a forked process."""

    def run(self, result=None):
        if result is None: result = unittest.TestResult()
        run_isolated(unittest.TestSuite, self, result)


def run_isolated(klass, self, result):
    """Run a test suite or case in a subprocess, using the run method on klass.
    """
    c2pread, c2pwrite = os.pipe()
    # fixme - error -> result
    # now fork
    pid = os.fork()
    if pid == 0:
        # Child
        # Close parent's pipe ends
        os.close(c2pread)
        # Dup fds for child
        os.dup2(c2pwrite, 1)
        # Close pipe fds.
        os.close(c2pwrite)

        # at this point, sys.stdin is redirected, now we want
        # to filter it to escape ]'s.
        ### XXX: test and write that bit.

        result = TestProtocolClient(sys.stdout)
        klass.run(self, result)
        sys.stdout.flush()
        sys.stderr.flush()
        # exit HARD, exit NOW.
        os._exit(0)
    else:
        # Parent
        # Close child pipe ends
        os.close(c2pwrite)
        # hookup a protocol engine
        protocol = TestProtocolServer(result)
        protocol.readFrom(os.fdopen(c2pread, 'rU'))
        os.waitpid(pid, 0)
        # TODO return code evaluation.
    return result
